@@ -51,7 +51,9 @@ public:
 		const std::string& feedurl);
 	void update_rssitem_unread_and_enqueued(RssItem* item,
 		const std::string& feedurl);
-	void cleanup_cache(std::vector<std::shared_ptr<RssFeed>> feeds,
+	/// if requested, removes unreachable data stored in cache.
+	/// returns a count of unreachable feeds and items.
+	std::uint64_t cleanup_cache(std::vector<std::shared_ptr<RssFeed>> feeds,
 		bool always_clean = false);
 	void do_vacuum();
 	std::vector<std::shared_ptr<RssItem>> search_for_items(
@@ -70,7 +72,6 @@ public:
 		time_t t,
 		const std::string& etag);
 	void mark_item_deleted(const std::string& guid, bool b);
-	void mark_feed_items_deleted(const std::string& feedurl);
 	void remove_old_deleted_items(RssFeed* feed);
 	void mark_items_read_by_guid(const std::vector<std::string>& guids);
 	std::vector<std::string> get_read_item_guids();
