@@ -502,8 +502,9 @@ int Controller::run(const CliArgsParser& args)
 	v->set_tags(tags);
 	v->set_cache(rsscache);
 
-	if (args.cmds_to_execute().has_value()) {
-		execute_commands(args.cmds_to_execute().value());
+	const auto cmds_to_execute = args.cmds_to_execute();
+	if (cmds_to_execute.size() >= 1) {
+		execute_commands(cmds_to_execute);
 		return EXIT_SUCCESS;
 	}
 
@@ -532,9 +533,15 @@ int Controller::run(const CliArgsParser& args)
 		std::cout.flush();
 	}
 	try {
-		rsscache->cleanup_cache(feedcontainer.get_all_feeds());
+		const std::uint64_t amt = rsscache->cleanup_cache(
+				feedcontainer.get_all_feeds());
 		if (!args.silent()) {
 			std::cout << _("done.") << std::endl;
+			if (amt > 0u) {
+				std::cout << _("Unreachable feeds found, consider setting "
+						"`cleanup-on-quit yes` or run `newsboat --cleanup`")
+					<< std::endl;
+			}
 		}
 	} catch (const DbException& e) {
 		LOG(Level::USERERROR, "Cleaning up cache failed: %s", e.what());
@@ -562,7 +569,7 @@ void Controller::mark_all_read(const std::string& feedurl)
 	try {
 		rsscache->mark_all_read(feedurl);
 	} catch (const DbException& e) {
-		v->show_error(strprintf::fmt(
+		v->get_statusline().show_error(strprintf::fmt(
 				_("Error: couldn't mark all feeds read: %s"),
 				e.what()));
 		return;
@@ -628,6 +635,14 @@ void Controller::mark_all_read(unsigned int pos)
 	feedcontainer.mark_all_feed_items_read(feed);
 }
 
+void Controller::mark_all_read(const std::vector<std::string>& item_guids)
+{
+	ScopeMeasure m("Controller::mark_all_read");
+	if (api) {
+		api->mark_articles_read(item_guids);
+	}
+}
+
 void Controller::replace_feed(std::shared_ptr<RssFeed> oldfeed,
 	std::shared_ptr<RssFeed> newfeed,
 	unsigned int pos,
@@ -648,7 +663,26 @@ void Controller::replace_feed(std::shared_ptr<RssFeed> oldfeed,
 	feed->set_tags(urlcfg->get_tags(oldfeed->rssurl()));
 	feed->set_order(oldfeed->get_order());
 	feedcontainer.replace_feed(pos, feed);
-	queueManager.autoenqueue(feed);
+
+	if (cfg.get_configvalue_as_bool("podcast-auto-enqueue")) {
+		const auto result = queueManager.autoenqueue(feed);
+		switch (result.status) {
+		case EnqueueStatus::QUEUED_SUCCESSFULLY:
+		case EnqueueStatus::URL_QUEUED_ALREADY:
+			// All is well, nothing to be done
+			break;
+		case EnqueueStatus::OUTPUT_FILENAME_USED_ALREADY:
+			v->get_statusline().show_error(
+				strprintf::fmt(_("Generated filename (%s) is used already."),
+					result.extra_info));
+			break;
+		case EnqueueStatus::QUEUE_FILE_OPEN_ERROR:
+			v->get_statusline().show_error(
+				strprintf::fmt(_("Failed to open queue file: %s."), result.extra_info));
+			break;
+		}
+	}
+
 	for (const auto& item : feed->items()) {
 		rsscache->update_rssitem_unread_and_enqueued(item, feed->rssurl());
 	}
@@ -669,9 +703,12 @@ int Controller::import_opml(const std::string& opmlFile,
 		return EXIT_FAILURE;
 	}
 
-	if (!opml::import(opmlFile, urlReader)) {
+	const auto import_error = opml::import(opmlFile, urlReader);
+	if (import_error.has_value()) {
 		std::cout << strprintf::fmt(
-				_("An error occurred while parsing %s."), opmlFile)
+				_("An error occurred while parsing %s: %s"),
+				opmlFile,
+				import_error.value())
 			<< std::endl;
 		return EXIT_FAILURE;
 	} else {
@@ -722,17 +759,17 @@ std::vector<std::shared_ptr<RssItem>> Controller::search_for_items(
 	return items;
 }
 
-void Controller::enqueue_url(std::shared_ptr<RssItem> item,
+EnqueueResult Controller::enqueue_url(std::shared_ptr<RssItem> item,
 	std::shared_ptr<RssFeed> feed)
 {
-	queueManager.enqueue_url(item, feed);
+	return queueManager.enqueue_url(item, feed);
 }
 
 void Controller::reload_urls_file()
 {
 	const auto error_message = urlcfg->reload();
 	if (error_message.has_value()) {
-		v->set_status(error_message.value());
+		v->get_statusline().show_message(error_message.value());
 		return;
 	}
 
@@ -908,7 +945,7 @@ void Controller::update_config()
 				strprintf::fmt("Couldn't open %s: %s",
 					cfg.get_configvalue("error-log"),
 					e.what());
-			v->show_error(msg);
+			v->get_statusline().show_error(msg);
 			std::cerr << msg << std::endl;
 		}
 	}
@@ -919,7 +956,7 @@ void Controller::load_configfile(const std::string& filename)
 	if (cfgparser.parse_file(filename)) {
 		update_config();
 	} else {
-		v->show_error(strprintf::fmt(
+		v->get_statusline().show_error(strprintf::fmt(
 				_("Error: couldn't open configuration file `%s'!"),
 				filename));
 	}
