@@ -4,6 +4,7 @@
 #include <cassert>
 #include <cstring>
 #include <langinfo.h>
+#include <numeric>
 #include <sstream>
 #include <string>
 
@@ -47,7 +48,7 @@ void FeedListFormAction::init()
 {
 	set_keymap_hints();
 
-	f.run(-3); // compute all widget dimensions
+	recalculate_widget_dimensions();
 
 	if (v->get_ctrl()->get_refresh_on_start()) {
 		v->get_ctrl()->get_reloader()->start_reload_all_thread();
@@ -108,7 +109,7 @@ REDO:
 				v->push_itemlist(pos);
 			} else {
 				// should not happen
-				v->show_error(_("No feed selected!"));
+				v->get_statusline().show_error(_("No feed selected!"));
 			}
 		}
 	}
@@ -120,7 +121,7 @@ REDO:
 		if (visible_feeds.size() > 0 && feedpos.length() > 0) {
 			v->get_ctrl()->get_reloader()->reload(pos);
 		} else {
-			v->show_error(
+			v->get_statusline().show_error(
 				_("No feed selected!")); // should not happen
 		}
 	}
@@ -210,17 +211,14 @@ REDO:
 		}
 	}
 	break;
-	case OP_OPENINBROWSER:
-		if (visible_feeds.size() > 0 && feedpos.length() > 0) {
-			std::shared_ptr<RssFeed> feed = v->get_ctrl()->get_feedcontainer()->get_feed(
-					pos);
-			if (feed) {
-				return open_feed_in_browser(feed);
-			}
-		} else {
-			v->show_error(_("No feed selected!"));
-		}
-		break;
+	case OP_OPENINBROWSER: {
+		const bool interactive = true;
+		return open_position_in_browser(pos, interactive);
+	}
+	case OP_OPENINBROWSER_NONINTERACTIVE: {
+		const bool interactive = false;
+		return open_position_in_browser(pos, interactive);
+	}
 	case OP_OPENALLUNREADINBROWSER:
 		if (visible_feeds.size() > 0 && feedpos.length() > 0) {
 			std::shared_ptr<RssFeed> feed =
@@ -238,16 +236,16 @@ REDO:
 				exit_code = open_unread_items_in_browser(feed, false);
 
 				if (!exit_code.has_value()) {
-					v->show_error(_("Failed to spawn browser"));
+					v->get_statusline().show_error(_("Failed to spawn browser"));
 					return false;
 				} else if (*exit_code != 0) {
-					v->show_error(strprintf::fmt(_("Browser returned error code %i"),
+					v->get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
 							*exit_code));
 					return false;
 				}
 			}
 		} else {
-			v->show_error(_("No feed selected!"));
+			v->get_statusline().show_error(_("No feed selected!"));
 		}
 		break;
 	case OP_OPENALLUNREADINBROWSER_AND_MARK:
@@ -268,10 +266,11 @@ REDO:
 				exit_code = open_unread_items_in_browser(feed, true);
 
 				if (!exit_code.has_value()) {
-					v->show_error(_("Failed to spawn browser"));
+					v->get_statusline().show_error(_("Failed to spawn browser"));
 					return false;
 				} else if (*exit_code != 0) {
-					v->show_error(strprintf::fmt(_("Browser returned error code %i"), *exit_code));
+					v->get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
+							*exit_code));
 					return false;
 				}
 
@@ -300,22 +299,24 @@ REDO:
 			"`%s'",
 			feedpos);
 		if (visible_feeds.size() > 0 && feedpos.length() > 0) {
-			v->set_status(_("Marking feed read..."));
 			try {
-				v->get_ctrl()->mark_all_read(pos);
-				do_redraw = true;
-				v->set_status("");
+				{
+					const auto message_lifetime = v->get_statusline().show_message_until_finished(
+							_("Marking feed read..."));
+					v->get_ctrl()->mark_all_read(pos);
+					do_redraw = true;
+				}
 				bool show_read = cfg->get_configvalue_as_bool("show-read-feeds");
 				if (visible_feeds.size() > (pos + 1) && show_read) {
 					list.set_position(pos + 1);
 				}
 			} catch (const DbException& e) {
-				v->show_error(strprintf::fmt(
+				v->get_statusline().show_error(strprintf::fmt(
 						_("Error: couldn't mark feed read: %s"),
 						e.what()));
 			}
 		} else {
-			v->show_error(
+			v->get_statusline().show_error(
 				_("No feed selected!")); // should not happen
 		}
 	}
@@ -336,7 +337,7 @@ REDO:
 		LOG(Level::INFO,
 			"FeedListFormAction: jumping to next unread feed");
 		if (!jump_to_next_unread_feed(local_tmp)) {
-			v->show_error(_("No feeds with unread items."));
+			v->get_statusline().show_error(_("No feeds with unread items."));
 		}
 	}
 	break;
@@ -345,7 +346,7 @@ REDO:
 		LOG(Level::INFO,
 			"FeedListFormAction: jumping to previous unread feed");
 		if (!jump_to_previous_unread_feed(local_tmp)) {
-			v->show_error(_("No feeds with unread items."));
+			v->get_statusline().show_error(_("No feeds with unread items."));
 		}
 	}
 	break;
@@ -353,7 +354,7 @@ REDO:
 		unsigned int local_tmp;
 		LOG(Level::INFO, "FeedListFormAction: jumping to next feed");
 		if (!jump_to_next_feed(local_tmp)) {
-			v->show_error(_("Already on last feed."));
+			v->get_statusline().show_error(_("Already on last feed."));
 		}
 	}
 	break;
@@ -362,7 +363,7 @@ REDO:
 		LOG(Level::INFO,
 			"FeedListFormAction: jumping to previous feed");
 		if (!jump_to_previous_feed(local_tmp)) {
-			v->show_error(_("Already on first feed."));
+			v->get_statusline().show_error(_("Already on first feed."));
 		}
 	}
 	break;
@@ -371,7 +372,7 @@ REDO:
 		LOG(Level::INFO,
 			"FeedListFormAction: jumping to random unread feed");
 		if (!jump_to_random_unread_feed(local_tmp)) {
-			v->show_error(_("No feeds with unread items."));
+			v->get_statusline().show_error(_("No feeds with unread items."));
 		}
 	}
 	break;
@@ -381,7 +382,8 @@ REDO:
 			v->confirm(_("Do you really want to mark all feeds as read (y:Yes n:No)? "),
 				_("yn")) == *_("y")) {
 			LOG(Level::INFO, "FeedListFormAction: marking all feeds read");
-			v->set_status(_("Marking all feeds read..."));
+			const auto message_lifetime = v->get_statusline().show_message_until_finished(
+					_("Marking all feeds read..."));
 			if (tag == "") {
 				v->get_ctrl()->mark_all_read("");
 			} else {
@@ -393,7 +395,6 @@ REDO:
 					v->get_ctrl()->mark_all_read(feedurl);
 				}
 			}
-			v->set_status("");
 			do_redraw = true;
 		}
 		break;
@@ -429,7 +430,7 @@ REDO:
 				filterhistory.add_line(newfilter);
 				if (newfilter.length() > 0) {
 					if (!matcher.parse(newfilter)) {
-						v->show_error(strprintf::fmt(
+						v->get_statusline().show_error(strprintf::fmt(
 								_("Error: couldn't "
 									"parse filter "
 									"command `%s': %s"),
@@ -443,7 +444,7 @@ REDO:
 				}
 			}
 		} else {
-			v->show_error(_("No filters defined."));
+			v->get_statusline().show_error(_("No filters defined."));
 		}
 		break;
 	case OP_SEARCH:
@@ -527,46 +528,59 @@ REDO:
 	return true;
 }
 
-bool FeedListFormAction::open_feed_in_browser(const std::shared_ptr<RssFeed>&
-	feed) const
+bool FeedListFormAction::open_position_in_browser(unsigned int pos,
+	bool interactive) const
 {
-	if (!feed->is_query_feed()) {
-		LOG(Level::INFO,
-			"FeedListFormAction: opening feed %s",
-			feed->link());
-
-		std::string url;
-		if (!feed->link().empty()) {
-			url = feed->link();
-		} else if (!feed->rssurl().empty()) {
-			url = feed->rssurl();
-		} else {
-			// rssurl can't be empty, so if we got to this branch,
-			// something is clearly wrong with Newsboat internals.
-			// That's why we write a message to the log, and not
-			// just display it to the user.
-			LOG(Level::INFO,
-				"FeedListFormAction: cannot open feed in browser "
-				"because both `link' and `rssurl' fields are "
-				"empty");
-		}
-
-		if (!url.empty()) {
-			const std::string feedurl = feed->rssurl();
-			const auto exit_code = v->open_in_browser(url, feedurl);
-			if (!exit_code.has_value()) {
-				v->show_error(_("Failed to spawn browser"));
-				return false;
-			} else if (*exit_code != 0) {
-				v->show_error(strprintf::fmt(_("Browser returned error code %i"), *exit_code));
-				return false;
-			}
-		}
-		return true;
-	} else {
-		v->show_error(_("Cannot open query feeds in the browser!"));
+	if (visible_feeds.empty()) {
+		v->get_statusline().show_error(_("No feed selected!"));
 		return false;
 	}
+
+	std::shared_ptr<RssFeed> feed = v->get_ctrl()->get_feedcontainer()->get_feed(
+			pos);
+	if (feed == nullptr) {
+		v->get_statusline().show_error(_("No feed selected!"));
+		return false;
+	}
+
+	if (feed->is_query_feed()) {
+		v->get_statusline().show_error(_("Cannot open query feeds in the browser!"));
+		return false;
+	}
+
+	LOG(Level::INFO, "FeedListFormAction: opening feed %s, interactive: %s",
+		feed->link(),
+		interactive ? "true" : "false");
+
+	std::string url;
+	if (!feed->link().empty()) {
+		url = feed->link();
+	} else if (!feed->rssurl().empty()) {
+		url = feed->rssurl();
+	} else {
+		// rssurl can't be empty, so if we got to this branch,
+		// something is clearly wrong with Newsboat internals.
+		// That's why we write a message to the log, and not
+		// just display it to the user.
+		LOG(Level::INFO,
+			"FeedListFormAction: cannot open feed in browser "
+			"because both `link' and `rssurl' fields are "
+			"empty");
+	}
+
+	if (!url.empty()) {
+		const std::string feedurl = feed->rssurl();
+		const auto exit_code = v->open_in_browser(url, feedurl, interactive);
+		if (!exit_code.has_value()) {
+			v->get_statusline().show_error(_("Failed to spawn browser"));
+			return false;
+		} else if (*exit_code != 0) {
+			v->get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
+					*exit_code));
+			return false;
+		}
+	}
+	return true;
 }
 
 void FeedListFormAction::update_visible_feeds(
@@ -929,10 +943,11 @@ void FeedListFormAction::update_form_title(unsigned int width)
 	fmt.register_fmt('N', PROGRAM_NAME);
 	fmt.register_fmt('V', utils::program_version());
 	fmt.register_fmt('u', std::to_string(count_unread_feeds()));
+	fmt.register_fmt('U', std::to_string(count_unread_articles()));
 	fmt.register_fmt('t', std::to_string(visible_feeds.size()));
 	fmt.register_fmt('F', apply_filter ? matcher.get_expression() : "");
 
-	f.set("head", fmt.do_format(title_format, width));
+	set_value("head", fmt.do_format(title_format, width));
 }
 
 unsigned int FeedListFormAction::count_unread_feeds()
@@ -945,13 +960,22 @@ unsigned int FeedListFormAction::count_unread_feeds()
 	});
 }
 
+unsigned int FeedListFormAction::count_unread_articles()
+{
+	unsigned int total = 0;
+	for (const auto& feed : visible_feeds) {
+		total += feed.first->unread_item_count();
+	}
+	return total;
+}
+
 void FeedListFormAction::op_end_setfilter()
 {
 	std::string filtertext = qna_responses[0];
 	filterhistory.add_line(filtertext);
 	if (filtertext.length() > 0) {
 		if (!matcher.parse(filtertext)) {
-			v->show_error(
+			v->get_statusline().show_error(
 				_("Error: couldn't parse filter command!"));
 		} else {
 			save_filterpos();
@@ -969,28 +993,28 @@ void FeedListFormAction::op_start_search()
 		"`%s'",
 		searchphrase);
 	if (searchphrase.length() > 0) {
-		v->set_status(_("Searching..."));
+		const auto message_lifetime = v->get_statusline().show_message_until_finished(
+				_("Searching..."));
 		searchhistory.add_line(searchphrase);
 		std::vector<std::shared_ptr<RssItem>> items;
 		try {
-			std::string utf8searchphrase = utils::convert_text(
-					searchphrase, "utf-8", nl_langinfo(CODESET));
+			const auto utf8searchphrase = utils::locale_to_utf8(searchphrase);
 			items = v->get_ctrl()->search_for_items(
 					utf8searchphrase, nullptr);
 		} catch (const DbException& e) {
-			v->show_error(strprintf::fmt(
+			v->get_statusline().show_error(strprintf::fmt(
 					_("Error while searching for `%s': %s"),
 					searchphrase,
 					e.what()));
 			return;
 		}
 		if (!items.empty()) {
-			std::shared_ptr<RssFeed> search_dummy_feed(new RssFeed(cache));
+			std::shared_ptr<RssFeed> search_dummy_feed(new RssFeed(cache, ""));
 			search_dummy_feed->set_search_feed(true);
 			search_dummy_feed->add_items(items);
 			v->push_searchresult(search_dummy_feed, searchphrase);
 		} else {
-			v->show_error(_("No results."));
+			v->get_statusline().show_error(_("No results."));
 		}
 	}
 }
@@ -1001,12 +1025,12 @@ void FeedListFormAction::handle_cmdline_num(unsigned int idx)
 		idx <= (visible_feeds[visible_feeds.size() - 1].second + 1)) {
 		int i = get_pos(idx - 1);
 		if (i == -1) {
-			v->show_error(_("Position not visible!"));
+			v->get_statusline().show_error(_("Position not visible!"));
 		} else {
 			list.set_position(i);
 		}
 	} else {
-		v->show_error(_("Invalid position!"));
+		v->get_statusline().show_error(_("Invalid position!"));
 	}
 }
 
