@@ -22,6 +22,7 @@ HtmlRenderer::HtmlRenderer(bool raw)
 {
 	tags["a"] = HtmlTag::A;
 	tags["embed"] = HtmlTag::EMBED;
+	tags["iframe"] = HtmlTag::IFRAME;
 	tags["br"] = HtmlTag::BR;
 	tags["pre"] = HtmlTag::PRE;
 	tags["ituneshack"] = HtmlTag::ITUNESHACK;
@@ -29,6 +30,7 @@ HtmlRenderer::HtmlRenderer(bool raw)
 	tags["blockquote"] = HtmlTag::BLOCKQUOTE;
 	tags["aside"] = HtmlTag::BLOCKQUOTE;
 	tags["p"] = HtmlTag::P;
+	tags["div"] = HtmlTag::DIV;
 	tags["h1"] = HtmlTag::H1;
 	tags["h2"] = HtmlTag::H2;
 	tags["h3"] = HtmlTag::H3;
@@ -107,6 +109,7 @@ void HtmlRenderer::render(std::istream& input,
 	unsigned int video_count = 0;
 	unsigned int audio_count = 0;
 	unsigned int source_count = 0;
+	unsigned int iframe_count = 0;
 	std::string curline;
 	int indent_level = 0;
 	std::vector<HtmlTag> list_elements_stack;
@@ -241,6 +244,48 @@ void HtmlRenderer::render(std::istream& input,
 			}
 			break;
 
+			case HtmlTag::IFRAME: {
+				std::string iframe_url;
+				std::string iframe_title;
+				try {
+					iframe_url = xpp.get_attribute_value("src");
+				} catch (const std::invalid_argument&) {
+					LOG(Level::WARN,
+						"HtmlRenderer::render: "
+						"found iframe tag without src attribute");
+					iframe_url = "";
+				}
+				try {
+					iframe_title = xpp.get_attribute_value("title");
+				} catch (const std::invalid_argument&) {
+					iframe_title = "";
+				}
+				if (iframe_url.length() > 0) {
+					add_nonempty_line(curline, tables, lines);
+					if (lines.size() > 0) {
+						std::string::size_type last_line_len =
+							lines[lines.size() - 1]
+							.second.length();
+						if (last_line_len >
+							static_cast<unsigned int>(
+								indent_level * 2)) {
+							add_line("", tables, lines);
+						}
+					}
+					prepare_new_line(curline,
+						tables.size() ? 0 : indent_level);
+
+					iframe_count++;
+					add_media_link(curline, links, url, iframe_url,
+						iframe_title, iframe_count, LinkType::IFRAME);
+
+					add_line(curline, tables, lines);
+					prepare_new_line(curline,
+						tables.size() ? 0 : indent_level);
+				}
+			}
+			break;
+
 			case HtmlTag::BR:
 				add_line(curline, tables, lines);
 				prepare_new_line(curline,
@@ -259,49 +304,35 @@ void HtmlRenderer::render(std::istream& input,
 				break;
 
 			case HtmlTag::IMG: {
-				std::string imgurl;
-				std::string imgtitle;
+				std::string img_url;
+				std::string img_label;
 				try {
-					imgurl = xpp.get_attribute_value("src");
+					img_url = xpp.get_attribute_value("src");
 				} catch (const std::invalid_argument&) {
 					LOG(Level::WARN,
-						"HtmlRenderer::render: found "
-						"img "
-						"tag with no src attribute");
-					imgurl = "";
+						"HtmlRenderer::render: "
+						"found img tag with no src "
+						"attribute");
+					img_url = "";
 				}
+				// Prefer `alt' over `title'
 				try {
-					imgtitle = xpp.get_attribute_value(
-							"title");
+					img_label = xpp.get_attribute_value("alt");
 				} catch (const std::invalid_argument&) {
-					imgtitle = "";
+					img_label = "";
 				}
-				if (imgurl.length() > 0) {
-					if (imgurl.substr(0, 5) == "data:") {
-						link_num = add_link(links,
-								"inline image",
-								LinkType::IMG);
-					} else {
-						link_num = add_link(links,
-								utils::censor_url(
-									utils::absolute_url(
-										url,
-										imgurl)),
-								LinkType::IMG);
-					}
-					if (imgtitle != "") {
-						curline.append(strprintf::fmt(
-								"[%s %u: %s]",
-								_("image"),
-								link_num,
-								imgtitle));
-					} else {
-						curline.append(strprintf::fmt(
-								"[%s %u]",
-								_("image"),
-								link_num));
+				if (img_label.empty()) {
+					try {
+						img_label = xpp.get_attribute_value("title");
+					} catch (const std::invalid_argument&) {
+						img_label = "";
 					}
+				}
+				if (!img_url.empty()) {
 					image_count++;
+					add_media_link(curline, links, url,
+						img_url, img_label, image_count,
+						LinkType::IMG);
 				}
 			}
 			break;
@@ -320,7 +351,8 @@ void HtmlRenderer::render(std::istream& input,
 			case HtmlTag::H4:
 			case HtmlTag::H5:
 			case HtmlTag::H6:
-			case HtmlTag::P: {
+			case HtmlTag::P:
+			case HtmlTag::DIV: {
 				add_nonempty_line(curline, tables, lines);
 				if (lines.size() > 0) {
 					std::string::size_type last_line_len =
@@ -528,92 +560,122 @@ void HtmlRenderer::render(std::istream& input,
 			}
 
 			case HtmlTag::VIDEO: {
-				std::string videourl;
+				std::string video_url;
 
+				// Decrement the appropriate counter if the
+				// previous media element had no sources
 				if (inside_video && source_count == 0) {
 					video_count--;
 				}
-
 				if (inside_audio && source_count == 0) {
 					audio_count--;
 				}
 
+				// "Pop" the previous media element if it didn't
+				// have a closing tag
 				if (inside_video || inside_audio) {
 					source_count = 0;
 					inside_video = false;
 					inside_audio = false;
 					LOG(Level::WARN,
-						"HtmlRenderer::render media element left unclosed");
+						"HtmlRenderer::render: "
+						"media element left unclosed");
 				}
+				inside_video = true;
+
+				// We will decrement this counter later if the
+				// current <video> element contains no sources,
+				// either by its `src' attribute or any child
+				// <source> elements
+				video_count++;
 
 				try {
-					videourl = xpp.get_attribute_value("src");
-					source_count++;
+					video_url = xpp.get_attribute_value("src");
 				} catch (const std::invalid_argument&) {
-					videourl = "";
+					video_url = "";
 				}
 
-				video_count++;
-				inside_video = true;
-				add_media_link(curline, links, url, videourl,
-					video_count, LinkType::VIDEO);
+				if (!video_url.empty()) {
+					// Video source retrieved from `src'
+					// attribute
+					source_count++;
+					add_media_link(curline, links, url,
+						video_url, "", video_count,
+						LinkType::VIDEO);
+				}
 			}
 			break;
 
 			case HtmlTag::AUDIO: {
-				std::string audiourl;
+				std::string audio_url;
 
+				// Decrement the appropriate counter if the
+				// previous media element had no sources
 				if (inside_video && source_count == 0) {
 					video_count--;
 				}
-
 				if (inside_audio && source_count == 0) {
 					audio_count--;
 				}
 
+				// "Pop" the previous media element if it didn't
+				// have a closing tag
 				if (inside_video || inside_audio) {
 					source_count = 0;
 					inside_video = false;
 					inside_audio = false;
 					LOG(Level::WARN,
-						"HtmlRenderer::render media element left unclosed");
+						"HtmlRenderer::render: "
+						"media element left unclosed");
 				}
+				inside_audio = true;
+
+				// We will decrement this counter later if the
+				// current <audio> element contains no sources,
+				// either by its `src' attribute or any child
+				// <source> elements
+				audio_count++;
 
 				try {
-					audiourl = xpp.get_attribute_value("src");
-					source_count++;
+					audio_url = xpp.get_attribute_value("src");
 				} catch (const std::invalid_argument&) {
-					audiourl = "";
+					audio_url = "";
 				}
 
-				audio_count++;
-				inside_audio = true;
-				add_media_link(curline, links, url, audiourl,
-					audio_count, LinkType::AUDIO);
+				if (!audio_url.empty()) {
+					// Audio source retrieved from `src'
+					// attribute
+					source_count++;
+					add_media_link(curline, links, url,
+						audio_url, "", audio_count,
+						LinkType::AUDIO);
+				}
 			}
 			break;
 
 			case HtmlTag::SOURCE: {
-				std::string sourceurl;
+				std::string source_url;
 
 				try {
-					sourceurl = xpp.get_attribute_value("src");
-					source_count++;
+					source_url = xpp.get_attribute_value("src");
 				} catch (const std::invalid_argument&) {
 					LOG(Level::WARN,
-						"HtmlRenderer::render: found source tag with no src attribute");
-					sourceurl = "";
+						"HtmlRenderer::render: "
+						"found source tag with no src "
+						"attribute");
+					source_url = "";
 				}
 
-				if (inside_video) {
+				if (inside_video && !source_url.empty()) {
+					source_count++;
 					add_media_link(curline, links, url,
-						sourceurl, video_count,
+						source_url, "", video_count,
 						LinkType::VIDEO);
 				}
-
-				if (inside_audio) {
+				if (inside_audio && !source_url.empty()) {
+					source_count++;
 					add_media_link(curline, links, url,
-						sourceurl, audio_count,
+						source_url, "", audio_count,
 						LinkType::AUDIO);
 				}
 			}
@@ -718,6 +780,7 @@ void HtmlRenderer::render(std::istream& input,
 			case HtmlTag::H5:
 			case HtmlTag::H6:
 			case HtmlTag::P:
+			case HtmlTag::DIV:
 				add_nonempty_line(curline, tables, lines);
 				prepare_new_line(curline,
 					tables.size() ? 0 : indent_level);
@@ -768,6 +831,7 @@ void HtmlRenderer::render(std::istream& input,
 				break;
 
 			case HtmlTag::EMBED:
+			case HtmlTag::IFRAME:
 			case HtmlTag::BR:
 			case HtmlTag::ITUNESHACK:
 			case HtmlTag::IMG:
@@ -979,19 +1043,30 @@ void HtmlRenderer::render(std::istream& input,
 
 void HtmlRenderer::add_media_link(std::string& curline,
 	std::vector<LinkPair>& links, const std::string& url,
-	const std::string& media_url, unsigned int media_count, LinkType type)
+	const std::string& media_url, const std::string& media_title,
+	unsigned int media_count, LinkType type)
 {
-	if (media_url.empty()) {
-		return;
+	std::string link_url;
+
+	// Display media_url except if it's an inline image
+	if (type == LinkType::IMG && media_url.substr(0, 5) == "data:") {
+		link_url = "inline image";
+	} else {
+		link_url = utils::censor_url(utils::absolute_url(url, media_url));
 	}
 
-	std::string type_str = type2str(type);
-	unsigned int link_num = add_link(links,
-			utils::censor_url(utils::absolute_url(url, media_url)),
-			type);
+	const std::string type_str = type2str(type);
+	const unsigned int link_num = add_link(links, link_url, type);
+	std::string output;
 
-	curline.append(strprintf::fmt("[%s %u (%s #%u)]", _(type_str.c_str()),
-			media_count, _("link"), link_num));
+	if (!media_title.empty()) {
+		output = strprintf::fmt("[%s %u: %s (%s #%u)]", _(type_str.c_str()),
+				media_count, media_title, _("link"), link_num);
+	} else {
+		output = strprintf::fmt("[%s %u (%s #%u)]", _(type_str.c_str()),
+				media_count, _("link"), link_num);
+	}
+	curline.append(output);
 }
 
 std::string HtmlRenderer::render_hr(const unsigned int width)
@@ -1012,6 +1087,8 @@ std::string HtmlRenderer::type2str(LinkType type)
 		return _("image");
 	case LinkType::EMBED:
 		return _("embedded flash");
+	case LinkType::IFRAME:
+		return _("iframe");
 	case LinkType::VIDEO:
 		return _("video");
 	case LinkType::AUDIO:
