@@ -74,7 +74,7 @@ bool ItemListFormAction::process_operation(Operation op,
 				show_searchresult ? search_phrase : "");
 			invalidate(itempos);
 		} else {
-			v->show_error(
+			v->get_statusline().show_error(
 				_("No item selected!")); // should not happen
 		}
 	}
@@ -97,28 +97,32 @@ bool ItemListFormAction::process_operation(Operation op,
 			}
 			invalidate(itempos);
 		} else {
-			v->show_error(
+			v->get_statusline().show_error(
 				_("No item selected!")); // should not happen
 		}
 	}
 	break;
 	case OP_DELETE_ALL: {
-		ScopeMeasure m1("OP_DELETE_ALL");
-		if (visible_items.size() > 0) {
-			if (!feed->is_search_feed()) {
-				v->get_ctrl()->mark_all_read(pos);
-			} else {
-				v->get_ctrl()->mark_all_read(feed);
-			}
+		if (!cfg->get_configvalue_as_bool("confirm-delete-all-articles") ||
+			v->confirm(_("Do you really want to delete all articles (y:Yes n:No)? "),
+				_("yn")) == *_("y")) {
+			ScopeMeasure m1("OP_DELETE_ALL");
+
+			std::vector<std::string> item_guids;
 			for (const auto& pair : visible_items) {
-				pair.first->set_deleted(true);
+				const auto item = pair.first;
+				item_guids.push_back(item->guid());
 			}
-			if (feed->is_query_feed() || feed->is_search_feed()) {
-				for (const auto& pair : visible_items) {
-					rsscache->mark_item_deleted(pair.first->guid(), true);
-				}
-			} else {
-				rsscache->mark_feed_items_deleted(feed->rssurl());
+			v->get_ctrl()->mark_all_read(item_guids);
+
+			for (const auto& pair : visible_items) {
+				const auto item = pair.first;
+
+				// mark as read
+				item->set_unread(false);
+				// mark as deleted
+				item->set_deleted(true);
+				rsscache->mark_item_deleted(item->guid(), true);
 			}
 			invalidate_list();
 		}
@@ -131,38 +135,34 @@ bool ItemListFormAction::process_operation(Operation op,
 	}
 	break;
 	case OP_OPENBROWSER_AND_MARK: {
-		LOG(Level::INFO, "ItemListFormAction: opening item at pos `%u'", itempos);
-		if (!visible_items.empty() && itempos < visible_items.size()) {
-			auto item = visible_items[itempos].first;
-			if (!open_item_in_browser(item)) {
-				return false;
-			}
-			item->set_unread(false);
-			v->get_ctrl()->mark_article_read(item->guid(), true);
-			if (cfg->get_configvalue_as_bool("openbrowser-and-mark-jumps-to-next-unread")) {
-				process_operation(OP_NEXTUNREAD);
-			} else {
-				if (itempos < visible_items.size() - 1) {
-					list.set_position(itempos + 1);
-				}
-			}
-			invalidate(itempos);
+		const bool interactive = true;
+		invalidate(itempos);
+		if (!open_position_in_browser(itempos, interactive)) {
+			return false;
+		}
+
+		auto item = visible_items[itempos].first;
+		item->set_unread(false);
+		v->get_ctrl()->mark_article_read(item->guid(), true);
+		if (cfg->get_configvalue_as_bool("openbrowser-and-mark-jumps-to-next-unread")) {
+			process_operation(OP_NEXTUNREAD);
 		} else {
-			v->show_error(_("No item selected!"));
+			if (itempos < visible_items.size() - 1) {
+				list.set_position(itempos + 1);
+			}
 		}
 	}
 	break;
 	case OP_OPENINBROWSER: {
-		LOG(Level::INFO, "ItemListFormAction: opening item at pos `%u'", itempos);
-		if (!visible_items.empty() && itempos < visible_items.size()) {
-			auto item = visible_items[itempos].first;
-			if (!open_item_in_browser(item)) {
-				return false;
-			}
-			invalidate(itempos);
-		} else {
-			v->show_error(_("No item selected!"));
-		}
+		const bool interactive = true;
+		invalidate(itempos);
+		return open_position_in_browser(itempos, interactive);
+	}
+	break;
+	case OP_OPENINBROWSER_NONINTERACTIVE: {
+		const bool interactive = false;
+		invalidate(itempos);
+		return open_position_in_browser(itempos, interactive);
 	}
 	break;
 	case OP_OPENALLUNREADINBROWSER: {
@@ -175,13 +175,14 @@ bool ItemListFormAction::process_operation(Operation op,
 			// We can't just `const auto exit_code = ...` here because this
 			// triggers -Wmaybe-initialized in GCC 9 with -O2.
 			nonstd::optional<std::uint8_t> exit_code;
-			exit_code = open_unread_items_in_browser(feed, true);
+			exit_code = open_unread_items_in_browser(feed, false);
 
 			if (!exit_code.has_value()) {
-				v->show_error(_("Failed to spawn browser"));
+				v->get_statusline().show_error(_("Failed to spawn browser"));
 				return false;
 			} else if (*exit_code != 0) {
-				v->show_error(strprintf::fmt(_("Browser returned error code %i"), *exit_code));
+				v->get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
+						*exit_code));
 				return false;
 			}
 		}
@@ -200,10 +201,11 @@ bool ItemListFormAction::process_operation(Operation op,
 			exit_code = open_unread_items_in_browser(feed, true);
 
 			if (!exit_code.has_value()) {
-				v->show_error(_("Failed to spawn browser"));
+				v->get_statusline().show_error(_("Failed to spawn browser"));
 				return false;
 			} else if (*exit_code != 0) {
-				v->show_error(strprintf::fmt(_("Browser returned error code %i"), *exit_code));
+				v->get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
+						*exit_code));
 				return false;
 			}
 			invalidate_list();
@@ -213,8 +215,9 @@ bool ItemListFormAction::process_operation(Operation op,
 	case OP_TOGGLEITEMREAD: {
 		LOG(Level::INFO, "ItemListFormAction: toggling item read at pos `%u'", itempos);
 		if (!visible_items.empty()) {
-			v->set_status(_("Toggling read flag for article..."));
 			try {
+				const auto message_lifetime = v->get_statusline().show_message_until_finished(
+						_("Toggling read flag for article..."));
 				if (automatic && args->size() > 0) {
 					if ((*args)[0] == "read") {
 						visible_items[itempos]
@@ -233,7 +236,6 @@ bool ItemListFormAction::process_operation(Operation op,
 							.first->guid(),
 							false);
 					}
-					v->set_status("");
 				} else {
 					// mark as undeleted
 					visible_items[itempos]
@@ -251,10 +253,9 @@ bool ItemListFormAction::process_operation(Operation op,
 						visible_items[itempos]
 						.first->guid(),
 						unread);
-					v->set_status("");
 				}
 			} catch (const DbException& e) {
-				v->set_status(strprintf::fmt(
+				v->get_statusline().show_error(strprintf::fmt(
 						_("Error while toggling read flag: %s"),
 						e.what()));
 			}
@@ -298,7 +299,7 @@ bool ItemListFormAction::process_operation(Operation op,
 					if (!links.empty()) {
 						v->push_urlview(links, feed);
 					} else {
-						v->show_error(
+						v->get_statusline().show_error(
 							_("URL list empty."));
 					}
 				} else {
@@ -308,7 +309,7 @@ bool ItemListFormAction::process_operation(Operation op,
 				}
 			}
 		} else {
-			v->show_error(
+			v->get_statusline().show_error(
 				_("No item selected!")); // should not happen
 		}
 		break;
@@ -337,7 +338,7 @@ bool ItemListFormAction::process_operation(Operation op,
 				}
 			}
 		} else {
-			v->show_error(
+			v->get_statusline().show_error(
 				_("No item selected!")); // should not happen
 		}
 	}
@@ -363,7 +364,7 @@ bool ItemListFormAction::process_operation(Operation op,
 				}
 			}
 		} else {
-			v->show_error(
+			v->get_statusline().show_error(
 				_("No item selected!")); // should not happen
 		}
 	}
@@ -384,7 +385,7 @@ bool ItemListFormAction::process_operation(Operation op,
 			}
 			save_article(filename, item);
 		} else {
-			v->show_error(_("Error: no item selected!"));
+			v->get_statusline().show_error(_("Error: no item selected!"));
 		}
 	}
 	break;
@@ -401,7 +402,7 @@ bool ItemListFormAction::process_operation(Operation op,
 			v->get_ctrl()->get_reloader()->reload(pos);
 			invalidate_list();
 		} else {
-			v->show_error(
+			v->get_statusline().show_error(
 				_("Error: you can't reload search results."));
 		}
 		break;
@@ -423,7 +424,7 @@ bool ItemListFormAction::process_operation(Operation op,
 			"ItemListFormAction: jumping to next unread item");
 		if (!jump_to_next_unread_item(false)) {
 			if (!v->get_next_unread(*this)) {
-				v->show_error(_("No unread items."));
+				v->get_statusline().show_error(_("No unread items."));
 			}
 		}
 		break;
@@ -432,7 +433,7 @@ bool ItemListFormAction::process_operation(Operation op,
 			"ItemListFormAction: jumping to previous unread item");
 		if (!jump_to_previous_unread_item(false)) {
 			if (!v->get_previous_unread(*this)) {
-				v->show_error(_("No unread items."));
+				v->get_statusline().show_error(_("No unread items."));
 			}
 		}
 		break;
@@ -440,7 +441,7 @@ bool ItemListFormAction::process_operation(Operation op,
 		LOG(Level::INFO, "ItemListFormAction: jumping to next item");
 		if (!jump_to_next_item(false)) {
 			if (!v->get_next(*this)) {
-				v->show_error(_("Already on last item."));
+				v->get_statusline().show_error(_("Already on last item."));
 			}
 		}
 		break;
@@ -449,41 +450,42 @@ bool ItemListFormAction::process_operation(Operation op,
 			"ItemListFormAction: jumping to previous item");
 		if (!jump_to_previous_item(false)) {
 			if (!v->get_previous(*this)) {
-				v->show_error(_("Already on first item."));
+				v->get_statusline().show_error(_("Already on first item."));
 			}
 		}
 		break;
 	case OP_RANDOMUNREAD:
 		if (!jump_to_random_unread_item()) {
 			if (!v->get_random_unread(*this)) {
-				v->show_error(_("No unread items."));
+				v->get_statusline().show_error(_("No unread items."));
 			}
 		}
 		break;
 	case OP_NEXTUNREADFEED:
 		if (!v->get_next_unread_feed(*this)) {
-			v->show_error(_("No unread feeds."));
+			v->get_statusline().show_error(_("No unread feeds."));
 		}
 		break;
 	case OP_PREVUNREADFEED:
 		if (!v->get_prev_unread_feed(*this)) {
-			v->show_error(_("No unread feeds."));
+			v->get_statusline().show_error(_("No unread feeds."));
 		}
 		break;
 	case OP_NEXTFEED:
 		if (!v->get_next_feed(*this)) {
-			v->show_error(_("Already on last feed."));
+			v->get_statusline().show_error(_("Already on last feed."));
 		}
 		break;
 	case OP_PREVFEED:
 		if (!v->get_prev_feed(*this)) {
-			v->show_error(_("Already on first feed."));
+			v->get_statusline().show_error(_("Already on first feed."));
 		}
 		break;
 	case OP_MARKFEEDREAD:
 		LOG(Level::INFO, "ItemListFormAction: marking feed read");
-		v->set_status(_("Marking feed read..."));
 		try {
+			const auto message_lifetime = v->get_statusline().show_message_until_finished(
+					_("Marking feed read..."));
 			if (feed->rssurl() != "") {
 				v->get_ctrl()->mark_all_read(pos);
 			} else {
@@ -526,17 +528,17 @@ bool ItemListFormAction::process_operation(Operation op,
 				}
 			}
 			invalidate_list();
-			v->set_status("");
 		} catch (const DbException& e) {
-			v->show_error(strprintf::fmt(
+			v->get_statusline().show_error(strprintf::fmt(
 					_("Error: couldn't mark feed read: %s"),
 					e.what()));
 		}
 		break;
-	case OP_MARKALLABOVEASREAD:
+	case OP_MARKALLABOVEASREAD: {
 		LOG(Level::INFO,
 			"ItemListFormAction: marking all above as read");
-		v->set_status(_("Marking all above as read..."));
+		const auto message_lifetime = v->get_statusline().show_message_until_finished(
+				_("Marking all above as read..."));
 		if (itempos < visible_items.size()) {
 			for (unsigned int i = 0; i < itempos; ++i) {
 				if (visible_items[i].first->unread()) {
@@ -553,8 +555,8 @@ bool ItemListFormAction::process_operation(Operation op,
 			}
 			invalidate_list();
 		}
-		v->set_status("");
 		break;
+	}
 	case OP_TOGGLESHOWREAD:
 		LOG(Level::DEBUG,
 			"ItemListFormAction: toggling show-read-articles");
@@ -582,7 +584,7 @@ bool ItemListFormAction::process_operation(Operation op,
 					qna, OP_PIPE_TO, &cmdlinehistory);
 			}
 		} else {
-			v->show_error(_("No item selected!"));
+			v->get_statusline().show_error(_("No item selected!"));
 		}
 		break;
 	case OP_SEARCH: {
@@ -634,7 +636,7 @@ bool ItemListFormAction::process_operation(Operation op,
 				filterhistory.add_line(newfilter);
 				if (newfilter.length() > 0) {
 					if (!matcher.parse(newfilter)) {
-						v->show_error(strprintf::fmt(
+						v->get_statusline().show_error(strprintf::fmt(
 								_("Error: couldn't "
 									"parse filter "
 									"command `%s': %s"),
@@ -648,7 +650,7 @@ bool ItemListFormAction::process_operation(Operation op,
 				}
 			}
 		} else {
-			v->show_error(_("No filters defined."));
+			v->get_statusline().show_error(_("No filters defined."));
 		}
 
 		break;
@@ -766,20 +768,31 @@ bool ItemListFormAction::process_operation(Operation op,
 	return true;
 }
 
-bool ItemListFormAction::open_item_in_browser(
-	const std::shared_ptr<RssItem>& item) const
+bool ItemListFormAction::open_position_in_browser(
+	unsigned int pos, bool interactive) const
 {
-	const auto link = item->link();
-	const auto feedurl = item->feedurl();
-	const auto exit_code = v->open_in_browser(link, feedurl);
-	if (!exit_code.has_value()) {
-		v->show_error(_("Failed to spawn browser"));
-		return false;
-	} else if (*exit_code != 0) {
-		v->show_error(strprintf::fmt(_("Browser returned error code %i"), *exit_code));
+	LOG(Level::INFO,
+		"ItemListFormAction: opening item at pos `%u', interactive: %s",
+		pos,
+		interactive ? "true" : "false");
+	if (!visible_items.empty() && pos < visible_items.size()) {
+		const auto item = visible_items[pos].first;
+		const auto link = item->link();
+		const auto feedurl = item->feedurl();
+		const auto exit_code = v->open_in_browser(link, feedurl, interactive);
+		if (!exit_code.has_value()) {
+			v->get_statusline().show_error(_("Failed to spawn browser"));
+			return false;
+		} else if (*exit_code != 0) {
+			v->get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
+					*exit_code));
+			return false;
+		}
+		return true;
+	} else {
+		v->get_statusline().show_error(_("No item selected!"));
 		return false;
 	}
-	return true;
 }
 
 void ItemListFormAction::finished_qna(Operation op)
@@ -836,7 +849,7 @@ void ItemListFormAction::qna_end_setfilter()
 
 	if (filtertext.length() > 0) {
 		if (!matcher.parse(filtertext)) {
-			v->show_error(
+			v->get_statusline().show_error(
 				_("Error: couldn't parse filter command!"));
 			return;
 		}
@@ -850,7 +863,7 @@ void ItemListFormAction::qna_end_setfilter()
 void ItemListFormAction::qna_end_editflags()
 {
 	if (visible_items.empty()) {
-		v->show_error(_("No item selected!")); // should not happen
+		v->get_statusline().show_error(_("No item selected!")); // should not happen
 		return;
 	}
 
@@ -858,7 +871,7 @@ void ItemListFormAction::qna_end_editflags()
 	if (itempos < visible_items.size()) {
 		visible_items[itempos].first->set_flags(qna_responses[0]);
 		v->get_ctrl()->update_flags(visible_items[itempos].first);
-		v->set_status(_("Flags updated."));
+		v->get_statusline().show_message(_("Flags updated."));
 		LOG(Level::DEBUG,
 			"ItemListFormAction::finished_qna: updated flags");
 		invalidate(itempos);
@@ -872,16 +885,16 @@ void ItemListFormAction::qna_start_search()
 		return;
 	}
 
-	v->set_status(_("Searching..."));
 	searchhistory.add_line(searchphrase);
 	std::vector<std::shared_ptr<RssItem>> items;
 	try {
-		std::string utf8searchphrase = utils::convert_text(
-				searchphrase, "utf-8", nl_langinfo(CODESET));
+		const auto message_lifetime = v->get_statusline().show_message_until_finished(
+				_("Searching..."));
+		const auto utf8searchphrase = utils::locale_to_utf8(searchphrase);
 		items = v->get_ctrl()->search_for_items(
 				utf8searchphrase, feed);
 	} catch (const DbException& e) {
-		v->show_error(
+		v->get_statusline().show_error(
 			strprintf::fmt(_("Error while searching for `%s': %s"),
 				searchphrase,
 				e.what()));
@@ -889,11 +902,11 @@ void ItemListFormAction::qna_start_search()
 	}
 
 	if (items.empty()) {
-		v->show_error(_("No results."));
+		v->get_statusline().show_error(_("No results."));
 		return;
 	}
 
-	std::shared_ptr<RssFeed> search_dummy_feed(new RssFeed(rsscache));
+	std::shared_ptr<RssFeed> search_dummy_feed(new RssFeed(rsscache, ""));
 	search_dummy_feed->set_search_feed(true);
 	search_dummy_feed->add_items(items);
 
@@ -993,7 +1006,7 @@ void ItemListFormAction::prepare()
 	try {
 		do_update_visible_items();
 	} catch (MatcherException& e) {
-		v->show_error(strprintf::fmt(
+		v->get_statusline().show_error(strprintf::fmt(
 				_("Error: applying the filter failed: %s"), e.what()));
 		return;
 	}
@@ -1114,9 +1127,9 @@ void ItemListFormAction::goto_item(const std::string& title)
 
 void ItemListFormAction::init()
 {
-	f.run(-3); // FRUN - compute all widget dimensions
+	recalculate_widget_dimensions();
 	list.set_position(0);
-	f.set("msg", "");
+	set_value("msg", "");
 	set_keymap_hints();
 	invalidate_list();
 	do_update_visible_items();
@@ -1168,7 +1181,7 @@ void ItemListFormAction::set_head(const std::string& s,
 				cfg->get_configvalue("searchresult-title-format"),
 				width);
 	}
-	f.set("head", title);
+	set_value("head", title);
 }
 
 bool ItemListFormAction::jump_to_previous_unread_item(bool start_with_last)
@@ -1305,12 +1318,12 @@ void ItemListFormAction::handle_cmdline_num(unsigned int idx)
 		idx <= visible_items[visible_items.size() - 1].second + 1) {
 		int i = get_pos(idx - 1);
 		if (i == -1) {
-			v->show_error(_("Position not visible!"));
+			v->get_statusline().show_error(_("Position not visible!"));
 		} else {
 			list.set_position(i);
 		}
 	} else {
-		v->show_error(_("Invalid position!"));
+		v->get_statusline().show_error(_("Invalid position!"));
 	}
 }
 
@@ -1326,11 +1339,11 @@ void ItemListFormAction::handle_cmdline(const std::string& cmd)
 		}
 		if (tokens[0] == "save") {
 			if (tokens.size() < 2) {
-				v->show_error(_("Error: no filename provided"));
+				v->get_statusline().show_error(_("Error: no filename provided"));
 				return;
 			}
 			if (visible_items.empty()) {
-				v->show_error(_("Error: no item selected!"));
+				v->get_statusline().show_error(_("Error: no item selected!"));
 				return;
 			}
 			const std::string filename = utils::resolve_tilde(tokens[1]);
@@ -1377,14 +1390,14 @@ void ItemListFormAction::save_article(const std::string& filename,
 	std::shared_ptr<RssItem> item)
 {
 	if (filename == "") {
-		v->show_error(_("Aborted saving."));
+		v->get_statusline().show_error(_("Aborted saving."));
 	} else {
 		try {
 			v->get_ctrl()->write_item(item, filename);
-			v->set_status(strprintf::fmt(
+			v->get_statusline().show_message(strprintf::fmt(
 					_("Saved article to %s"), filename));
 		} catch (...) {
-			v->show_error(strprintf::fmt(
+			v->get_statusline().show_error(strprintf::fmt(
 					_("Error: couldn't save article to %s"),
 					filename));
 		}
