@@ -45,15 +45,15 @@ ItemViewFormAction::~ItemViewFormAction() {}
 
 void ItemViewFormAction::init()
 {
-	f.set("msg", "");
+	set_value("msg", "");
 	do_redraw = true;
 	links.clear();
 	num_lines = 0;
 	if (!cfg->get_configvalue_as_bool("display-article-progress")) {
-		f.set("percentwidth", "0");
+		set_value("percentwidth", "0");
 	} else {
 		const size_t min_field_width = 6;
-		f.set("percentwidth",
+		set_value("percentwidth",
 			std::to_string(
 		std::max({
 			min_field_width,
@@ -93,7 +93,7 @@ void ItemViewFormAction::prepare()
 		{
 			ScopeMeasure("itemview::prepare: rendering");
 			// XXX HACK: render once so that we get a proper widget width
-			f.run(-3);
+			recalculate_widget_dimensions();
 		}
 
 		update_head(item);
@@ -141,7 +141,7 @@ void ItemViewFormAction::prepare()
 		}
 
 		textview.stfl_replace_lines(num_lines, formatted_text);
-		f.set("article_offset", "0");
+		set_value("article_offset", "0");
 
 		if (in_search) {
 			rxman.remove_last_regex("article");
@@ -173,7 +173,7 @@ bool ItemViewFormAction::process_operation(Operation op,
 			v->get_ctrl()->mark_article_read(item->guid(), true);
 		}
 	} catch (const DbException& e) {
-		v->show_error(strprintf::fmt(
+		v->get_statusline().show_error(strprintf::fmt(
 				_("Error while marking article as read: %s"),
 				e.what()));
 	}
@@ -205,13 +205,33 @@ bool ItemViewFormAction::process_operation(Operation op,
 	case OP_ENQUEUE: {
 		if (item->enclosure_url().length() > 0 &&
 			utils::is_http_url(item->enclosure_url())) {
-			v->get_ctrl()->enqueue_url(item, feed);
-			v->set_status(
-				strprintf::fmt(_("Added %s to download queue."),
-					item->enclosure_url()));
+			const EnqueueResult result = v->get_ctrl()->enqueue_url(item, feed);
+			rsscache->update_rssitem_unread_and_enqueued(item, feed->rssurl());
+			switch (result.status) {
+			case EnqueueStatus::QUEUED_SUCCESSFULLY:
+				v->get_statusline().show_message(
+					strprintf::fmt(_("Added %s to download queue."),
+						item->enclosure_url()));
+				return true;
+			case EnqueueStatus::URL_QUEUED_ALREADY:
+				v->get_statusline().show_message(
+					strprintf::fmt(_("%s is already queued."),
+						item->enclosure_url()));
+				return true; // Not a failure, just an idempotent action
+			case EnqueueStatus::OUTPUT_FILENAME_USED_ALREADY:
+				v->get_statusline().show_error(
+					strprintf::fmt(_("Generated filename (%s) is used already."),
+						result.extra_info));
+				return false;
+			case EnqueueStatus::QUEUE_FILE_OPEN_ERROR:
+				v->get_statusline().show_error(
+					strprintf::fmt(_("Failed to open queue file: %s."), result.extra_info));
+				return false;
+			}
 		} else {
-			v->set_status(strprintf::fmt(
+			v->get_statusline().show_error(strprintf::fmt(
 					_("Invalid URL: '%s'"), item->enclosure_url()));
+			return false;
 		}
 	}
 	break;
@@ -227,14 +247,14 @@ bool ItemViewFormAction::process_operation(Operation op,
 							item->title())));
 		}
 		if (filename == "") {
-			v->show_error(_("Aborted saving."));
+			v->get_statusline().show_error(_("Aborted saving."));
 		} else {
 			try {
 				v->get_ctrl()->write_item(item, filename);
-				v->set_status(strprintf::fmt(
+				v->get_statusline().show_message(strprintf::fmt(
 						_("Saved article to %s."), filename));
 			} catch (...) {
-				v->show_error(strprintf::fmt(
+				v->get_statusline().show_error(strprintf::fmt(
 						_("Error: couldn't write article to "
 							"file %s"),
 						filename));
@@ -245,7 +265,14 @@ bool ItemViewFormAction::process_operation(Operation op,
 	case OP_OPENINBROWSER:
 	case OP_OPENBROWSER_AND_MARK: {
 		LOG(Level::INFO, "ItemViewFormAction::process_operation: starting browser");
-		return open_link_in_browser(item->link());
+		const bool interactive = true;
+		return open_link_in_browser(item->link(), interactive);
+	}
+	break;
+	case OP_OPENINBROWSER_NONINTERACTIVE: {
+		LOG(Level::INFO, "ItemViewFormAction::process_operation: starting browser");
+		const bool interactive = false;
+		return open_link_in_browser(item->link(), interactive);
 	}
 	break;
 	case OP_BOOKMARK:
@@ -312,7 +339,7 @@ bool ItemViewFormAction::process_operation(Operation op,
 			if (links.size() > 0) {
 				v->push_urlview(links, feed);
 			} else {
-				v->show_error(_("URL list empty."));
+				v->get_statusline().show_error(_("URL list empty."));
 			}
 		} else {
 			qna_responses.clear();
@@ -334,7 +361,7 @@ bool ItemViewFormAction::process_operation(Operation op,
 			do_redraw = true;
 		} else {
 			v->pop_current_formaction();
-			v->show_error(_("No unread items."));
+			v->get_statusline().show_error(_("No unread items."));
 		}
 		break;
 	case OP_PREVUNREAD:
@@ -345,7 +372,7 @@ bool ItemViewFormAction::process_operation(Operation op,
 			do_redraw = true;
 		} else {
 			v->pop_current_formaction();
-			v->show_error(_("No unread items."));
+			v->get_statusline().show_error(_("No unread items."));
 		}
 		break;
 	case OP_NEXT:
@@ -355,7 +382,7 @@ bool ItemViewFormAction::process_operation(Operation op,
 			do_redraw = true;
 		} else {
 			v->pop_current_formaction();
-			v->show_error(_("Already on last item."));
+			v->get_statusline().show_error(_("Already on last item."));
 		}
 		break;
 	case OP_PREV:
@@ -365,7 +392,7 @@ bool ItemViewFormAction::process_operation(Operation op,
 			do_redraw = true;
 		} else {
 			v->pop_current_formaction();
-			v->show_error(_("Already on first item."));
+			v->get_statusline().show_error(_("Already on first item."));
 		}
 		break;
 	case OP_RANDOMUNREAD:
@@ -375,24 +402,25 @@ bool ItemViewFormAction::process_operation(Operation op,
 			do_redraw = true;
 		} else {
 			v->pop_current_formaction();
-			v->show_error(_("No unread items."));
+			v->get_statusline().show_error(_("No unread items."));
 		}
 		break;
-	case OP_TOGGLEITEMREAD:
+	case OP_TOGGLEITEMREAD: {
 		LOG(Level::INFO,
 			"ItemViewFormAction::process_operation: setting unread and quitting");
-		v->set_status(_("Toggling read flag for article..."));
+		const auto message_lifetime = v->get_statusline().show_message_until_finished(
+				_("Toggling read flag for article..."));
 		try {
 			item->set_unread(true);
 			v->get_ctrl()->mark_article_read(item->guid(), false);
 		} catch (const DbException& e) {
-			v->show_error(strprintf::fmt(
+			v->get_statusline().show_error(strprintf::fmt(
 					_("Error while marking article as unread: %s"),
 					e.what()));
 		}
-		v->set_status("");
 		quit = true;
 		break;
+	}
 	case OP_QUIT:
 		LOG(Level::INFO, "ItemViewFormAction::process_operation: quitting");
 		quit = true;
@@ -421,7 +449,8 @@ bool ItemViewFormAction::process_operation(Operation op,
 			op,
 			idx);
 		if (idx < links.size()) {
-			return open_link_in_browser(links[idx].first);
+			const bool interactive = true;
+			return open_link_in_browser(links[idx].first, interactive);
 		}
 	}
 	break;
@@ -463,15 +492,17 @@ bool ItemViewFormAction::process_operation(Operation op,
 	return true;
 }
 
-bool ItemViewFormAction::open_link_in_browser(const std::string& link) const
+bool ItemViewFormAction::open_link_in_browser(const std::string& link,
+	bool interactive) const
 {
 	const std::string feedurl = item->feedurl();
-	const auto exit_code = v->open_in_browser(link, feedurl);
+	const auto exit_code = v->open_in_browser(link, feedurl, interactive);
 	if (!exit_code.has_value()) {
-		v->show_error(_("Failed to spawn browser"));
+		v->get_statusline().show_error(_("Failed to spawn browser"));
 		return false;
 	} else if (*exit_code != 0) {
-		v->show_error(strprintf::fmt(_("Browser returned error code %i"), *exit_code));
+		v->get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
+				*exit_code));
 		return false;
 	}
 	return true;
@@ -512,7 +543,7 @@ void ItemViewFormAction::set_head(const std::string& s,
 
 	const unsigned int width = textview.get_width();
 
-	f.set("head",
+	set_value("head",
 		fmt.do_format(
 			cfg->get_configvalue("itemview-title-format"), width));
 }
@@ -525,16 +556,16 @@ void ItemViewFormAction::handle_cmdline(const std::string& cmd)
 			std::string filename = utils::resolve_tilde(tokens[1]);
 
 			if (filename == "") {
-				v->show_error(_("Aborted saving."));
+				v->get_statusline().show_error(_("Aborted saving."));
 			} else {
 				try {
 					v->get_ctrl()->write_item(
 						item, filename);
-					v->set_status(strprintf::fmt(
+					v->get_statusline().show_message(strprintf::fmt(
 							_("Saved article to %s"),
 							filename));
 				} catch (...) {
-					v->show_error(strprintf::fmt(
+					v->get_statusline().show_error(strprintf::fmt(
 							_("Error: couldn't save "
 								"article to %s"),
 							filename));
@@ -555,7 +586,7 @@ void ItemViewFormAction::finished_qna(Operation op)
 	case OP_INT_EDITFLAGS_END:
 		item->set_flags(qna_responses[0]);
 		v->get_ctrl()->update_flags(item);
-		v->set_status(_("Flags updated."));
+		v->get_statusline().show_message(_("Flags updated."));
 		do_redraw = true;
 		break;
 	case OP_INT_START_SEARCH:
@@ -581,7 +612,8 @@ void ItemViewFormAction::finished_qna(Operation op)
 		unsigned int idx = 0;
 		sscanf(qna_responses[0].c_str(), "%u", &idx);
 		if (idx && idx - 1 < links.size()) {
-			open_link_in_browser(links[idx - 1].first);
+			const bool interactive = true;
+			open_link_in_browser(links[idx - 1].first, interactive);
 		}
 	}
 	break;
@@ -624,11 +656,11 @@ void ItemViewFormAction::update_percent()
 			percent);
 
 		if (offset == 0 || percent == 0) {
-			f.set("percent", _("Top"));
+			set_value("percent", _("Top"));
 		} else if (offset == (num_lines - 1)) {
-			f.set("percent", _("Bottom"));
+			set_value("percent", _("Bottom"));
 		} else {
-			f.set("percent", strprintf::fmt("%3u %% ", percent));
+			set_value("percent", strprintf::fmt("%3u %% ", percent));
 		}
 	}
 }
@@ -687,7 +719,7 @@ void ItemViewFormAction::highlight_text(const std::string& searchphrase)
 			"ItemViewFormAction::highlight_text: handle_action "
 			"failed, error = %s",
 			e.what());
-		v->show_error(_("Error: invalid regular expression!"));
+		v->get_statusline().show_error(_("Error: invalid regular expression!"));
 	}
 }
 
