@@ -55,8 +55,7 @@ std::shared_ptr<RssFeed> RssParser::parse()
 		return nullptr;
 	}
 
-	std::shared_ptr<RssFeed> feed(new RssFeed(ch));
-	feed->set_rssurl(my_uri);
+	std::shared_ptr<RssFeed> feed(new RssFeed(ch, my_uri));
 
 	/*
 	 * After parsing is done, we fill our feed object with title,
@@ -169,9 +168,8 @@ void RssParser::retrieve_uri(const std::string& uri)
 	} else if (utils::is_exec_url(uri)) {
 		get_execplugin(uri.substr(5, uri.length() - 5));
 	} else if (utils::is_filter_url(uri)) {
-		std::string filter, url;
-		utils::extract_filter(uri, filter, url);
-		download_filterplugin(filter, url);
+		const auto parts = utils::extract_filter(uri);
+		download_filterplugin(std::string(parts.script_name), std::string(parts.url));
 	} else if (utils::is_query_url(my_uri)) {
 		f.rss_version = rsspp::Feed::Version::UNKNOWN;
 	} else if (my_uri.substr(0, 7) == "file://") {
