@@ -44,8 +44,6 @@ GCRY_THREAD_OPTION_PTHREAD_IMPL;
 #include <openssl/crypto.h>
 #endif
 
-#include "rs_utils.h"
-
 using HTTPMethod = newsboat::utils::HTTPMethod;
 
 namespace newsboat {
@@ -83,7 +81,7 @@ void append_escapes(std::string& str, char c)
 
 std::string utils::strip_comments(const std::string& line)
 {
-	return RustString(rs_strip_comments(line.c_str()));
+	return std::string(utils::bridged::strip_comments(line));
 }
 
 std::vector<std::string> utils::tokenize_quoted(const std::string& str,
@@ -214,8 +212,7 @@ std::vector<std::string> utils::tokenize_spaced(const std::string& str,
 
 std::string utils::consolidate_whitespace(const std::string& str)
 {
-
-	return RustString(rs_consolidate_whitespace(str.c_str()));
+	return std::string(utils::bridged::consolidate_whitespace(str));
 }
 
 std::vector<std::string> utils::tokenize_nl(const std::string& str,
@@ -259,155 +256,29 @@ std::vector<std::string> utils::tokenize_nl(const std::string& str,
 	return tokens;
 }
 
-std::string utils::translit(const std::string& tocode,
-	const std::string& fromcode)
+std::string utils::translit(const std::string& tocode, const std::string& fromcode)
 {
-	std::string tlit = "//TRANSLIT";
-
-	enum class TranslitState { UNKNOWN, SUPPORTED, UNSUPPORTED };
-
-	static TranslitState state = TranslitState::UNKNOWN;
-
-	// TRANSLIT is not needed when converting to unicode encodings
-	if (tocode == "utf-8" || tocode == "WCHAR_T") {
-		return tocode;
-	}
-
-	if (state == TranslitState::UNKNOWN) {
-		iconv_t cd = ::iconv_open(
-				(tocode + "//TRANSLIT").c_str(), fromcode.c_str());
-
-		if (cd == reinterpret_cast<iconv_t>(-1)) {
-			if (errno == EINVAL) {
-				iconv_t cd = ::iconv_open(
-						tocode.c_str(), fromcode.c_str());
-				if (cd != reinterpret_cast<iconv_t>(-1)) {
-					state = TranslitState::UNSUPPORTED;
-				} else {
-					fprintf(stderr,
-						"iconv_open('%s', '%s') "
-						"failed: %s",
-						tocode.c_str(),
-						fromcode.c_str(),
-						strerror(errno));
-					abort();
-				}
-			} else {
-				fprintf(stderr,
-					"iconv_open('%s//TRANSLIT', '%s') "
-					"failed: %s",
-					tocode.c_str(),
-					fromcode.c_str(),
-					strerror(errno));
-				abort();
-			}
-		} else {
-			state = TranslitState::SUPPORTED;
-		}
-
-		iconv_close(cd);
-	}
-
-	return ((state == TranslitState::SUPPORTED) ? (tocode + tlit)
-			: (tocode));
-}
-
-std::string utils::convert_text(const std::string& text,
-	const std::string& tocode,
-	const std::string& fromcode)
-{
-	std::string result;
-
-	if (strcasecmp(tocode.c_str(), fromcode.c_str()) == 0) {
-		return text;
-	}
-
-	iconv_t cd = ::iconv_open(
-			translit(tocode, fromcode).c_str(), fromcode.c_str());
-
-	if (cd == reinterpret_cast<iconv_t>(-1)) {
-		return result;
-	}
-
-	size_t inbytesleft;
-	size_t outbytesleft;
-
-	/*
-	 * of all the Unix-like systems around there, only Linux/glibc seems to
-	 * come with a SuSv3-conforming iconv implementation.
-	 */
-#if !defined(__linux__) && !defined(__GLIBC__) && !defined(__APPLE__) && \
-	!defined(__OpenBSD__) && !defined(__FreeBSD__) &&                \
-	!defined(__DragonFly__)
-	const char* inbufp;
-#else
-	char* inbufp;
-#endif
-	char outbuf[16];
-	char* outbufp = outbuf;
-
-	outbytesleft = sizeof(outbuf);
-	inbufp = const_cast<char*>(
-			text.c_str()); // evil, but spares us some trouble
-	inbytesleft = strlen(inbufp);
-
-	do {
-		char* old_outbufp = outbufp;
-		int rc = ::iconv(
-				cd, &inbufp, &inbytesleft, &outbufp, &outbytesleft);
-		if (-1 == rc) {
-			switch (errno) {
-			case E2BIG:
-				result.append(
-					old_outbufp, outbufp - old_outbufp);
-				outbufp = outbuf;
-				outbytesleft = sizeof(outbuf);
-				inbufp += strlen(inbufp) - inbytesleft;
-				inbytesleft = strlen(inbufp);
-				break;
-			case EILSEQ:
-			case EINVAL:
-				result.append(
-					old_outbufp, outbufp - old_outbufp);
-				result.append("?");
-				inbufp += strlen(inbufp) - inbytesleft + 1;
-				inbytesleft = strlen(inbufp);
-				break;
-			default:
-				break;
-			}
-		} else {
-			result.append(old_outbufp, outbufp - old_outbufp);
-		}
-	} while (inbytesleft > 0);
-
-	iconv_close(cd);
-
-	return result;
+	return std::string(utils::bridged::translit(tocode, fromcode));
 }
 
 std::string utils::utf8_to_locale(const std::string& text)
 {
-	if (text.empty()) {
-		return {};
-	}
+	const auto result = utils::bridged::utf8_to_locale(text);
+	return std::string(reinterpret_cast<const char*>(result.data()), result.size());
+}
 
-	return utils::convert_text(text, nl_langinfo(CODESET), "utf-8");
+std::string utils::locale_to_utf8(const std::string& text)
+{
+	const auto text_slice =
+		rust::Slice<unsigned char>(
+			reinterpret_cast<const unsigned char*>(text.c_str()),
+			text.length());
+	return std::string(utils::bridged::locale_to_utf8(text_slice));
 }
 
 std::string utils::get_command_output(const std::string& cmd)
 {
-	return RustString(rs_get_command_output(cmd.c_str()));
-}
-
-void utils::extract_filter(const std::string& line,
-	std::string& filter,
-	std::string& url)
-{
-	FilterUrl filterUrl = rs_extract_filter(line.c_str());
-
-	filter = RustString(filterUrl.filter);
-	url = RustString(filterUrl.url);
+	return std::string(utils::bridged::get_command_output(cmd));
 }
 
 static size_t my_write_data(void* buffer, size_t size, size_t nmemb,
@@ -479,10 +350,8 @@ std::string utils::retrieve_url(const std::string& url,
 	}
 
 	if (!authinfo.empty()) {
-		curl_easy_setopt(easyhandle,
-			CURLOPT_HTTPAUTH,
-			get_auth_method(
-				cfgcont->get_configvalue("http-auth-method")));
+		const auto auth_method = cfgcont->get_configvalue("http-auth-method");
+		curl_easy_setopt(easyhandle, CURLOPT_HTTPAUTH, get_auth_method(auth_method));
 		curl_easy_setopt(easyhandle, CURLOPT_USERPWD, authinfo.c_str());
 	}
 
@@ -505,32 +374,31 @@ std::string utils::retrieve_url(const std::string& url,
 	return buf;
 }
 
-void utils::run_command(const std::string& cmd, const std::string& input)
-{
-	rs_run_command(cmd.c_str(), input.c_str());
-}
-
 std::string utils::run_program(const char* argv[], const std::string& input)
 {
-	return RustString(rs_run_program(argv, input.c_str()));
+	rust::Vec<rust::String> rs_argv;
+	for (; *argv; ++argv) {
+		rs_argv.emplace_back(*argv);
+	}
+	return std::string(utils::bridged::run_program(rs_argv, input));
 }
 
 std::string utils::resolve_tilde(const std::string& str)
 {
-	return RustString(rs_resolve_tilde(str.c_str()));
+	return std::string(utils::bridged::resolve_tilde(str));
 }
 
 std::string utils::resolve_relative(const std::string& reference,
 	const std::string& fname)
 {
-	return RustString(rs_resolve_relative(reference.c_str(), fname.c_str()));
+	return std::string(utils::bridged::resolve_relative(reference, fname));
 }
 
 std::string utils::replace_all(std::string str,
 	const std::string& from,
 	const std::string& to)
 {
-	return RustString( rs_replace_all(str.c_str(), from.c_str(), to.c_str()) );
+	return std::string(utils::bridged::replace_all(str, from, to));
 }
 
 std::string utils::replace_all(const std::string& str,
@@ -581,7 +449,7 @@ std::string utils::wstr2str(const std::wstring& wstr)
 
 std::string utils::absolute_url(const std::string& url, const std::string& link)
 {
-	return RustString(rs_absolute_url(url.c_str(), link.c_str()));
+	return std::string(utils::bridged::absolute_url(url, link));
 }
 
 std::string utils::get_useragent(ConfigContainer* cfgcont)
@@ -618,16 +486,6 @@ unsigned int utils::to_u(const std::string& str,
 	return bridged::to_u(str, default_value);
 }
 
-bool utils::is_valid_color(const std::string& color)
-{
-	return rs_is_valid_color(color.c_str());
-}
-
-bool utils::is_valid_attribute(const std::string& attrib)
-{
-	return rs_is_valid_attribute(attrib.c_str());
-}
-
 std::vector<std::pair<unsigned int, unsigned int>> utils::partition_indexes(
 		unsigned int start,
 		unsigned int end,
@@ -647,26 +505,16 @@ std::vector<std::pair<unsigned int, unsigned int>> utils::partition_indexes(
 	return partitions;
 }
 
-size_t utils::strwidth(const std::string& str)
-{
-	return rs_strwidth(str.c_str());
-}
-
-size_t utils::strwidth_stfl(const std::string& str)
-{
-	return rs_strwidth_stfl(str.c_str());
-}
-
 std::string utils::substr_with_width(const std::string& str,
 	const size_t max_width)
 {
-	return RustString(rs_substr_with_width(str.c_str(), max_width));
+	return std::string(utils::bridged::substr_with_width(str, max_width));
 }
 
 std::string utils::substr_with_width_stfl(const std::string& str,
 	const size_t max_width)
 {
-	return RustString(rs_substr_with_width_stfl(str.c_str(), max_width));
+	return std::string(utils::bridged::substr_with_width_stfl(str, max_width));
 }
 
 std::string utils::join(const std::vector<std::string>& strings,
@@ -686,59 +534,34 @@ std::string utils::join(const std::vector<std::string>& strings,
 	return result;
 }
 
-bool utils::is_special_url(const std::string& url)
-{
-	return rs_is_special_url(url.c_str());
-}
-
-bool utils::is_http_url(const std::string& url)
-{
-	return rs_is_http_url(url.c_str());
-}
-
-bool utils::is_query_url(const std::string& url)
-{
-	return rs_is_query_url(url.c_str());
-}
-
-bool utils::is_filter_url(const std::string& url)
-{
-	return rs_is_filter_url(url.c_str());
-}
-
-bool utils::is_exec_url(const std::string& url)
-{
-	return rs_is_exec_url(url.c_str());
-}
-
 std::string utils::censor_url(const std::string& url)
 {
-	return RustString(rs_censor_url(url.c_str()));
+	return std::string(utils::bridged::censor_url(url));
 }
 
 std::string utils::quote_for_stfl(std::string str)
 {
-	return RustString(rs_quote_for_stfl(str.c_str()));
+	return std::string(utils::bridged::quote_for_stfl(str));
 }
 
 void utils::trim(std::string& str)
 {
-	str = RustString(rs_trim(str.c_str()));
+	str = std::string(utils::bridged::trim(str));
 }
 
 void utils::trim_end(std::string& str)
 {
-	str = RustString(rs_trim_end(str.c_str()));
+	str = std::string(utils::bridged::trim_end(str));
 }
 
 std::string utils::quote(const std::string& str)
 {
-	return RustString(rs_quote(str.c_str()));
+	return std::string(utils::bridged::quote(str));
 }
 
 std::string utils::quote_if_necessary(const std::string& str)
 {
-	return RustString(rs_quote_if_necessary(str.c_str()));
+	return std::string(utils::bridged::quote_if_necessary(str));
 }
 
 void utils::set_common_curl_options(CURL* handle, ConfigContainer* cfg)
@@ -831,12 +654,7 @@ std::string utils::get_content(xmlNode* node)
 
 std::string utils::get_basename(const std::string& url)
 {
-	return RustString(rs_get_basename(url.c_str()));
-}
-
-unsigned long utils::get_auth_method(const std::string& type)
-{
-	return rs_get_auth_method(type.c_str());
+	return std::string(utils::bridged::get_basename(url));
 }
 
 curl_proxytype utils::get_proxy_type(const std::string& type)
@@ -869,12 +687,13 @@ curl_proxytype utils::get_proxy_type(const std::string& type)
 
 std::string utils::unescape_url(const std::string& url)
 {
-	char* ptr = rs_unescape_url(url.c_str());
-	if (ptr == nullptr) {
+	bool success = false;
+	const auto result = utils::bridged::unescape_url(url, success);
+	if (!success) {
 		LOG(Level::DEBUG, "Rust failed to unescape url: %s", url );
 		throw std::runtime_error("unescaping url failed");
 	} else {
-		return RustString(ptr);
+		return std::string(result);
 	}
 }
 
@@ -888,31 +707,36 @@ std::wstring utils::clean_nonprintable_characters(std::wstring text)
 	return text;
 }
 
-unsigned int utils::gentabs(const std::string& str)
-{
-	return rs_gentabs(str.c_str());
-}
-
 /* Like mkdir(), but creates ancestors (parent directories) if they don't
  * exist. */
 int utils::mkdir_parents(const std::string& p, mode_t mode)
 {
-	return rs_mkdir_parents(p.c_str(), static_cast<std::uint32_t>(mode));
+	return utils::bridged::mkdir_parents(p, static_cast<std::uint32_t>(mode));
 }
 
 std::string utils::make_title(const std::string& const_url)
 {
-	return RustString(rs_make_title(const_url.c_str()));
+	return std::string(utils::bridged::make_title(const_url));
 }
 
 nonstd::optional<std::uint8_t> utils::run_interactively(
 	const std::string& command,
 	const std::string& caller)
 {
-	bool success = false;
-	const auto exit_code = rs_run_interactively(command.c_str(), caller.c_str(),
-			&success);
-	if (success) {
+	std::uint8_t exit_code = 0;
+	if (bridged::run_interactively(command, caller, exit_code)) {
+		return exit_code;
+	}
+
+	return nonstd::nullopt;
+}
+
+nonstd::optional<std::uint8_t> utils::run_non_interactively(
+	const std::string& command,
+	const std::string& caller)
+{
+	std::uint8_t exit_code = 0;
+	if (bridged::run_non_interactively(command, caller, exit_code)) {
 		return exit_code;
 	}
 
@@ -921,10 +745,11 @@ nonstd::optional<std::uint8_t> utils::run_interactively(
 
 std::string utils::getcwd()
 {
-	return RustString(rs_getcwd());
+	return std::string(utils::bridged::getcwd());
 }
 
-utils::ReadTextFileResult utils::read_text_file( const std::string& filename)
+nonstd::expected<std::vector<std::string>, utils::ReadTextFileError> utils::read_text_file(
+	const std::string& filename)
 {
 	rust::Vec<rust::String> c;
 	std::uint64_t error_line_number{};
@@ -943,11 +768,11 @@ utils::ReadTextFileResult utils::read_text_file( const std::string& filename)
 
 		if (error_line_number == 0) {
 			error.kind = ReadTextFileErrorKind::CantOpen;
-			error.message = strprintf::fmt(_("Failed to open file (%s)"),
+			error.message = strprintf::fmt(_("Failed to open file: %s"),
 					std::string(error_reason));
 		} else {
 			error.kind = ReadTextFileErrorKind::LineError;
-			error.message = strprintf::fmt(_("Failed to read line %u (%s)"),
+			error.message = strprintf::fmt(_("Failed to read line %u: %s"),
 					error_line_number, std::string(error_reason));
 		}
 
@@ -955,30 +780,23 @@ utils::ReadTextFileResult utils::read_text_file( const std::string& filename)
 	}
 }
 
-int utils::strnaturalcmp(const std::string& a, const std::string& b)
-{
-	return rs_strnaturalcmp(a.c_str(), b.c_str());
-}
-
 void utils::remove_soft_hyphens(std::string& text)
 {
-	/* Remove all soft-hyphens as they can behave unpredictably (see
-	 * https://github.com/akrennmair/newsbeuter/issues/259#issuecomment-259609490)
-	 * and inadvertently render as hyphens */
-	text = RustString(rs_remove_soft_hyphens(text.c_str()));
+	rust::String tmp(text);
+	utils::bridged::remove_soft_hyphens(tmp);
+	text = std::string(tmp);
 }
 
 bool utils::is_valid_podcast_type(const std::string& mimetype)
 {
-	return rs_is_valid_podcast_type(mimetype.c_str());
+	return utils::bridged::is_valid_podcast_type(mimetype);
 }
 
 nonstd::optional<LinkType> utils::podcast_mime_to_link_type(
 	const std::string& mimetype)
 {
-	bool ok = false;
-	const auto result = rs_podcast_mime_to_link_type(mimetype.c_str(), &ok);
-	if (ok) {
+	std::int64_t result = 0;
+	if (utils::bridged::podcast_mime_to_link_type(mimetype, result)) {
 		return static_cast<LinkType>(result);
 	}
 
@@ -1041,17 +859,12 @@ void utils::initialize_ssl_implementation(void)
 
 std::string utils::get_default_browser()
 {
-	return RustString(rs_get_default_browser());
+	return std::string(utils::bridged::get_default_browser());
 }
 
 std::string utils::program_version()
 {
-	return RustString(rs_program_version());
-}
-
-unsigned int utils::newsboat_version_major()
-{
-	return rs_newsboat_version_major();
+	return std::string(utils::bridged::program_version());
 }
 
 std::string utils::mt_strf_localtime(const std::string& format, time_t t)
