@@ -11,6 +11,7 @@
 #include <iomanip>
 #include <iostream>
 #include <libgen.h>
+#include <limits.h>
 #include <ncurses.h>
 #include <pwd.h>
 #include <string.h>
@@ -65,6 +66,7 @@ View::View(Controller* c)
 	, cfg(0)
 	, keys(0)
 	, current_formaction(0)
+	, status_line(*this)
 	, rxman(c->get_regexmanager())
 	, is_inside_qna(false)
 	, is_inside_cmdline(false)
@@ -110,9 +112,8 @@ void View::set_status(const std::string& msg)
 	auto fa = get_current_formaction();
 	if (fa != nullptr
 		&& std::dynamic_pointer_cast<EmptyFormAction>(fa) == nullptr) {
-		Stfl::Form& form = fa->get_form();
-		form.set("msg", msg);
-		form.run(-1);
+		fa->set_value("msg", msg);
+		fa->draw_form();
 	}
 }
 
@@ -121,6 +122,11 @@ void View::show_error(const std::string& msg)
 	set_status(msg);
 }
 
+StatusLine& View::get_statusline()
+{
+	return status_line;
+}
+
 bool View::run_commands(const std::vector<MacroCmd>& commands)
 {
 	for (auto command : commands) {
@@ -129,7 +135,7 @@ bool View::run_commands(const std::vector<MacroCmd>& commands)
 		}
 		std::shared_ptr<FormAction> fa = get_current_formaction();
 		fa->prepare();
-		fa->get_form().run(-1);
+		fa->draw_form();
 		if (!fa->process_op(command.op, true, &command.args)) {
 			// Operation failed, abort
 			return false;
@@ -156,7 +162,7 @@ int View::run()
 
 	if (!run_commands(keys->get_startup_operation_sequence())) {
 		Stfl::reset();
-		std::cerr << _("Error: Failed to execute startup commands") << std::endl;
+		std::cerr << _("Error: failed to execute startup commands") << std::endl;
 		return EXIT_FAILURE;
 	}
 
@@ -172,7 +178,7 @@ int View::run()
 		fa->prepare();
 
 		// we then receive the event and ignore timeouts.
-		const char* event_ptr = fa->get_form().run(60000);
+		const std::string event = fa->draw_form_wait_for_event(INT_MAX);
 
 		if (ctrl_c_hit) {
 			ctrl_c_hit = false;
@@ -187,13 +193,7 @@ int View::run()
 			}
 		}
 
-		if (!event_ptr) {
-			continue;
-		}
-
-		std::string event = event_ptr;
-
-		if (event == "TIMEOUT") {
+		if (event.empty() || event == "TIMEOUT") {
 			continue;
 		}
 
@@ -254,13 +254,13 @@ std::string View::run_modal(std::shared_ptr<FormAction> f,
 
 		fa->prepare();
 
-		const char* event = fa->get_form().run(1000);
+		const std::string event = fa->draw_form_wait_for_event(INT_MAX);
 		LOG(Level::DEBUG, "View::run: event = %s", event);
-		if (!event || strcmp(event, "TIMEOUT") == 0) {
+		if (event.empty() || event == "TIMEOUT") {
 			continue;
 		}
 
-		if (strcmp(event, "RESIZE") == 0) {
+		if (event == "RESIZE") {
 			handle_resize();
 			continue;
 		}
@@ -343,7 +343,7 @@ void View::open_in_pager(const std::string& filename)
 }
 
 nonstd::optional<std::uint8_t> View::open_in_browser(const std::string& url,
-	const std::string& feedurl)
+	const std::string& feedurl, bool interactive)
 {
 	std::string cmdline;
 	const std::string browser = cfg->get_configvalue("browser");
@@ -366,14 +366,19 @@ nonstd::optional<std::uint8_t> View::open_in_browser(const std::string& url,
 		cmdline.append(" " + escaped_url);
 	}
 
-	push_empty_formaction();
-	Stfl::reset();
-	const auto ret = utils::run_interactively(cmdline, "View::open_in_browser");
-	drop_queued_input();
-
-	pop_current_formaction();
-
-	return ret;
+	if (interactive) {
+		push_empty_formaction();
+		Stfl::reset();
+		const auto ret = utils::run_interactively(cmdline, "View::open_in_browser");
+		drop_queued_input();
+		pop_current_formaction();
+		return ret;
+	} else {
+		set_status(strprintf::fmt(_("Running browser: %s"), cmdline));
+		const auto ret = utils::run_non_interactively(cmdline, "View::open_in_browser");
+		set_status("");
+		return ret;
+	}
 }
 
 void View::update_visible_feeds(std::vector<std::shared_ptr<RssFeed>> feeds)
@@ -435,7 +440,7 @@ void View::push_searchresult(std::shared_ptr<RssFeed> feed,
 		formaction_stack.push_back(searchresult);
 		current_formaction = formaction_stack_size() - 1;
 	} else {
-		show_error(_("Error: feed contains no items!"));
+		status_line.show_error(_("Error: feed contains no items!"));
 	}
 }
 
@@ -460,7 +465,7 @@ std::shared_ptr<ItemListFormAction> View::push_itemlist(
 		current_formaction = formaction_stack_size() - 1;
 		return itemlist;
 	} else {
-		show_error(_("Error: feed contains no items!"));
+		status_line.show_error(_("Error: feed contains no items!"));
 		return nullptr;
 	}
 }
@@ -513,7 +518,7 @@ void View::push_itemview(std::shared_ptr<RssFeed> f,
 					item->guid(), true);
 			}
 		} catch (const DbException& e) {
-			show_error(strprintf::fmt(
+			status_line.show_error(strprintf::fmt(
 					_("Error while marking article as read: %s"),
 					e.what()));
 		}
@@ -573,24 +578,21 @@ void View::push_urlview(const std::vector<LinkPair>& links,
 	current_formaction = formaction_stack_size() - 1;
 }
 
-std::string View::run_filebrowser(const std::string& default_filename,
-	const std::string& dir)
+std::string View::run_filebrowser(const std::string& default_filename)
 {
 	auto filebrowser = std::make_shared<FileBrowserFormAction>(
 			this, filebrowser_str, cfg);
 	apply_colors(filebrowser);
-	filebrowser->set_dir(dir);
 	filebrowser->set_default_filename(default_filename);
 	filebrowser->set_parent_formaction(get_current_formaction());
 	return run_modal(filebrowser, "filenametext");
 }
 
-std::string View::run_dirbrowser(const std::string& dir)
+std::string View::run_dirbrowser()
 {
 	auto dirbrowser = std::make_shared<DirBrowserFormAction>(
 			this, filebrowser_str, cfg);
 	apply_colors(dirbrowser);
-	dirbrowser->set_dir(dir);
 	dirbrowser->set_parent_formaction(get_current_formaction());
 	return run_modal(dirbrowser, "filenametext");
 }
@@ -598,7 +600,7 @@ std::string View::run_dirbrowser(const std::string& dir)
 std::string View::select_tag()
 {
 	if (tags.size() == 0) {
-		show_error(_("No tags defined."));
+		status_line.show_error(_("No tags defined."));
 		return "";
 	}
 	auto selecttag = std::make_shared<SelectFormAction>(
@@ -630,17 +632,17 @@ char View::confirm(const std::string& prompt, const std::string& charset)
 	std::shared_ptr<FormAction> f = get_current_formaction();
 	// Push empty formaction so our "msg" is not overwritten
 	push_empty_formaction();
-	f->get_form().set("msg", prompt);
+	f->set_value("msg", prompt);
 
 	char result = 0;
 
 	do {
-		const char* event = f->get_form().run(0);
+		const std::string event = f->draw_form_wait_for_event(0);
 		LOG(Level::DEBUG, "View::confirm: event = %s", event);
-		if (!event) {
+		if (event.empty()) {
 			continue;
 		}
-		if (strcmp(event, "ESC") == 0 || strcmp(event, "ENTER") == 0) {
+		if (event == "ESC" || event == "ENTER") {
 			result = 0;
 			LOG(Level::DEBUG,
 				"View::confirm: user pressed ESC or ENTER, we "
@@ -654,8 +656,8 @@ char View::confirm(const std::string& prompt, const std::string& charset)
 			result);
 	} while (!result || strchr(charset.c_str(), result) == nullptr);
 
-	f->get_form().set("msg", "");
-	f->get_form().run(-1);
+	f->set_value("msg", "");
+	f->draw_form();
 
 	pop_current_formaction();
 
@@ -730,7 +732,7 @@ bool View::get_previous_unread(ItemListFormAction& itemlist,
 	} else if (cfg->get_configvalue_as_bool("goto-next-feed") == false) {
 		LOG(Level::DEBUG,
 			"View::get_previous_unread: goto-next-feed = false");
-		show_error(_("No unread items."));
+		status_line.show_error(_("No unread items."));
 	} else if (feedlist_form->jump_to_previous_unread_feed(feedpos)) {
 		LOG(Level::DEBUG,
 			"View::get_previous_unread: found feed with unread "
@@ -795,7 +797,7 @@ bool View::get_next_unread(ItemListFormAction& itemlist,
 	} else if (cfg->get_configvalue_as_bool("goto-next-feed") == false) {
 		LOG(Level::DEBUG,
 			"View::get_next_unread: goto-next-feed = false");
-		show_error(_("No unread items."));
+		status_line.show_error(_("No unread items."));
 	} else if (feedlist_form->jump_to_next_unread_feed(feedpos)) {
 		LOG(Level::DEBUG,
 			"View::get_next_unread: found feed with unread "
@@ -830,7 +832,7 @@ bool View::get_previous(ItemListFormAction& itemlist,
 		return true;
 	} else if (cfg->get_configvalue_as_bool("goto-next-feed") == false) {
 		LOG(Level::DEBUG, "View::get_previous: goto-next-feed = false");
-		show_error(_("Already on first item."));
+		status_line.show_error(_("Already on first item."));
 	} else if (feedlist_form->jump_to_previous_feed(feedpos)) {
 		LOG(Level::DEBUG, "View::get_previous: previous feed");
 		prepare_query_feed(feedlist_form->get_feed());
@@ -862,7 +864,7 @@ bool View::get_next(ItemListFormAction& itemlist, ItemViewFormAction* itemview)
 		return true;
 	} else if (cfg->get_configvalue_as_bool("goto-next-feed") == false) {
 		LOG(Level::DEBUG, "View::get_next: goto-next-feed = false");
-		show_error(_("Already on last item."));
+		status_line.show_error(_("Already on last item."));
 	} else if (feedlist_form->jump_to_next_feed(feedpos)) {
 		LOG(Level::DEBUG, "View::get_next: next feed");
 		prepare_query_feed(feedlist_form->get_feed());
@@ -929,7 +931,7 @@ void View::force_redraw()
 		&& std::dynamic_pointer_cast<EmptyFormAction>(fa) == nullptr) {
 		fa->set_redraw(true);
 		fa->prepare();
-		fa->get_form().run(-1);
+		fa->draw_form();
 	}
 }
 
@@ -967,8 +969,8 @@ void View::pop_current_formaction()
 		std::shared_ptr<FormAction> f = get_current_formaction();
 		if (f) {
 			f->set_redraw(true);
-			f->get_form().set("msg", "");
-			f->recalculate_form();
+			f->set_value("msg", "");
+			f->recalculate_widget_dimensions();
 		}
 	}
 }
@@ -1015,7 +1017,11 @@ void View::apply_colors(std::shared_ptr<FormAction> fa)
 {
 	LOG(Level::DEBUG, "View::apply_colors: fa = %s", fa->id());
 
-	colorman.apply_colors(fa->get_form());
+	const auto stfl_value_setter = [&](const std::string& name,
+	const std::string& value) {
+		fa->set_value(name, value);
+	};
+	colorman.apply_colors(stfl_value_setter);
 }
 
 void View::feedlist_mark_pos_if_visible(unsigned int pos)
@@ -1075,18 +1081,18 @@ void View::inside_cmdline(bool f)
 
 void View::clear_line(std::shared_ptr<FormAction> fa)
 {
-	fa->get_form().set("qna_value", "");
-	fa->get_form().set("qna_value_pos", "0");
+	fa->set_value("qna_value", "");
+	fa->set_value("qna_value_pos", "0");
 	LOG(Level::DEBUG, "View::clear_line: cleared line");
 }
 
 void View::clear_eol(std::shared_ptr<FormAction> fa)
 {
-	unsigned int pos = utils::to_u(fa->get_form().get("qna_value_pos"), 0);
-	std::string val = fa->get_form().get("qna_value");
+	unsigned int pos = utils::to_u(fa->get_value("qna_value_pos"), 0);
+	std::string val = fa->get_value("qna_value");
 	val.erase(pos, val.length());
-	fa->get_form().set("qna_value", val);
-	fa->get_form().set("qna_value_pos", std::to_string(val.length()));
+	fa->set_value("qna_value", val);
+	fa->set_value("qna_value_pos", std::to_string(val.length()));
 	LOG(Level::DEBUG, "View::clear_eol: cleared to end of line");
 }
 
@@ -1099,8 +1105,8 @@ void View::cancel_input(std::shared_ptr<FormAction> fa)
 void View::delete_word(std::shared_ptr<FormAction> fa)
 {
 	std::string::size_type curpos =
-		utils::to_u(fa->get_form().get("qna_value_pos"), 0);
-	std::string val = fa->get_form().get("qna_value");
+		utils::to_u(fa->get_value("qna_value_pos"), 0);
+	std::string val = fa->get_value("qna_value");
 	std::string::size_type firstpos = curpos;
 	LOG(Level::DEBUG, "View::delete_word: before val = %s", val);
 	if (firstpos >= val.length() || ::isspace(val[firstpos])) {
@@ -1119,8 +1125,8 @@ void View::delete_word(std::shared_ptr<FormAction> fa)
 	}
 	val.erase(firstpos, curpos - firstpos);
 	LOG(Level::DEBUG, "View::delete_word: after val = %s", val);
-	fa->get_form().set("qna_value", val);
-	fa->get_form().set("qna_value_pos", std::to_string(firstpos));
+	fa->set_value("qna_value", val);
+	fa->set_value("qna_value_pos", std::to_string(firstpos));
 }
 
 bool View::handle_qna_event(const std::string& event,
@@ -1155,7 +1161,7 @@ void View::handle_resize()
 	for (const auto& form : formaction_stack) {
 		if (form != nullptr) {
 			// Recalculate width and height of stfl widgets
-			form->get_form().run(-3);
+			form->recalculate_widget_dimensions();
 			form->set_redraw(true);
 		}
 	}
@@ -1163,7 +1169,7 @@ void View::handle_resize()
 
 void View::handle_cmdline_completion(std::shared_ptr<FormAction> fa)
 {
-	std::string fragment = fa->get_form().get("qna_value");
+	std::string fragment = fa->get_value("qna_value");
 	if (fragment != last_fragment || fragment == "") {
 		last_fragment = fragment;
 		suggestions = fa->get_suggestions(fragment);
@@ -1188,33 +1194,11 @@ void View::handle_cmdline_completion(std::shared_ptr<FormAction> fa)
 		suggestion = suggestions[(tab_count - 1) % suggestions.size()];
 		break;
 	}
-	fa->get_form().set("qna_value", suggestion);
-	fa->get_form().set(
-		"qna_value_pos", std::to_string(suggestion.length()));
+	fa->set_value("qna_value", suggestion);
+	fa->set_value("qna_value_pos", std::to_string(suggestion.length()));
 	last_fragment = suggestion;
 }
 
-void View::dump_current_form()
-{
-	std::string formtext =
-		formaction_stack[current_formaction]->get_form().dump(
-			"", "", 0);
-	time_t t = time(nullptr);
-	const auto fnbuf = utils::mt_strf_localtime(
-			"dumpform-%Y%m%d-%H%M%S.stfl",
-			t);
-	std::fstream f(fnbuf, std::ios_base::out);
-	if (!f.is_open()) {
-		show_error(strprintf::fmt("Error: couldn't open file %s: %s",
-				fnbuf,
-				strerror(errno)));
-		return;
-	}
-	f << formtext;
-	f.close();
-	set_status(strprintf::fmt("Dumped current form to file %s", fnbuf));
-}
-
 void View::ctrl_c_action(int /* sig */)
 {
 	LOG(Level::DEBUG, "caught SIGINT");
