@@ -60,7 +60,8 @@ TEST_CASE(
 				i == OP_MARKALLABOVEASREAD ||
 				i == OP_OPENALLUNREADINBROWSER_AND_MARK ||
 				i == OP_SAVEALL ||
-				i == OP_GOTO_TITLE) {
+				i == OP_GOTO_TITLE ||
+				i == OP_OPENINBROWSER_NONINTERACTIVE) {
 				continue;
 			}
 			bool used_in_some_context = false;
