@@ -1,6 +1,8 @@
 #include "utils.h"
 
+#include <algorithm>
 #include <chrono>
+#include <ctype.h>
 #include <fstream>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -9,7 +11,6 @@
 
 #include "3rd-party/catch.hpp"
 #include "htmlrenderer.h"
-#include "rs_utils.h"
 #include "test-helpers/chdir.h"
 #include "test-helpers/envvar.h"
 #include "test-helpers/stringmaker/optional.h"
@@ -531,34 +532,36 @@ TEST_CASE("get_command_output()", "[utils]")
 
 TEST_CASE("extract_filter()", "[utils]")
 {
-	std::string filter;
-	std::string url;
+	{
+		const auto parts =
+			utils::extract_filter("filter:~/bin/script.sh:https://newsboat.org");
+		REQUIRE(std::string(parts.script_name) == "~/bin/script.sh");
+		REQUIRE(std::string(parts.url) == "https://newsboat.org");
+	}
 
-	utils::extract_filter("filter:~/bin/script.sh:https://newsboat.org", filter,
-		url);
+	{
+		const auto parts = utils::extract_filter("filter::https://newsboat.org");
+		REQUIRE(std::string(parts.script_name) == "");
+		REQUIRE(std::string(parts.url) == "https://newsboat.org");
+	}
 
-	REQUIRE(filter == "~/bin/script.sh");
-	REQUIRE(url == "https://newsboat.org");
+	{
+		const auto parts = utils::extract_filter("filter:https://newsboat.org");
+		REQUIRE(std::string(parts.script_name) == "https");
+		REQUIRE(std::string(parts.url) == "//newsboat.org");
+	}
 
-	utils::extract_filter("filter::https://newsboat.org", filter, url);
+	{
+		const auto parts = utils::extract_filter("filter:foo:");
+		REQUIRE(std::string(parts.script_name) == "foo");
+		REQUIRE(std::string(parts.url) == "");
+	}
 
-	REQUIRE(filter == "");
-	REQUIRE(url == "https://newsboat.org");
-
-	utils::extract_filter("filter:https://newsboat.org", filter, url);
-
-	REQUIRE(filter == "https");
-	REQUIRE(url == "//newsboat.org");
-
-	utils::extract_filter("filter:foo:", filter, url);
-
-	REQUIRE(filter == "foo");
-	REQUIRE(url == "");
-
-	utils::extract_filter("filter:", filter, url);
-
-	REQUIRE(filter == "");
-	REQUIRE(url == "");
+	{
+		const auto parts = utils::extract_filter("filter:");
+		REQUIRE(std::string(parts.script_name) == "");
+		REQUIRE(std::string(parts.url) == "");
+	}
 }
 
 TEST_CASE("run_program()", "[utils]")
@@ -635,7 +638,7 @@ TEST_CASE("run_command() doesn't wait for the command to finish",
 	REQUIRE(runtime.count() < 60000);
 }
 
-TEST_CASE("resolve_tilde() replaces ~ with the path to the $HOME directory",
+TEST_CASE("resolve_tilde() replaces ~ with the path to the home directory",
 	"[utils]")
 {
 	TestHelpers::EnvVar envVar("HOME");
@@ -881,7 +884,8 @@ TEST_CASE("strwidth()", "[utils]")
 
 	REQUIRE(utils::strwidth("xx") == 2);
 
-	REQUIRE(utils::strwidth(utils::wstr2str(L"\uF91F")) == 2);
+	const auto input1 = utils::wstr2str(L"\uF91F");
+	REQUIRE(utils::strwidth(input1) == 2);
 	REQUIRE(utils::strwidth("\07") == 0);
 }
 
@@ -899,7 +903,8 @@ TEST_CASE("strwidth_stfl()", "[utils]")
 
 	REQUIRE(utils::strwidth_stfl("x<>y<>z") == 5);
 
-	REQUIRE(utils::strwidth_stfl(utils::wstr2str(L"\uF91F")) == 2);
+	const auto input1 = utils::wstr2str(L"\uF91F");
+	REQUIRE(utils::strwidth_stfl(input1) == 2);
 	REQUIRE(utils::strwidth_stfl("\07") == 0);
 }
 
@@ -1079,6 +1084,48 @@ TEST_CASE("run_interactively runs a command with inherited I/O", "[utils]")
 	// `nonstd::nullopt`, nor to test that it returns just the lower 8 bits.
 }
 
+TEST_CASE("run_non_interactively runs a command without I/O", "[utils]")
+{
+	SECTION("echo hello should return 0") {
+		const auto result = utils::run_non_interactively("echo hello", "test");
+		REQUIRE(result == 0);
+	}
+	SECTION("exit 1 should return 1") {
+		const auto result = utils::run_non_interactively("exit 1", "test");
+		REQUIRE(result == 1);
+	}
+
+	// Unfortunately, there is no easy way to provoke this function to return
+	// `nonstd::nullopt`, nor to test that it returns just the lower 8 bits.
+}
+
+TEST_CASE("run_non_interactively waits for program to finish", "[utils]")
+{
+	using namespace std::chrono;
+
+	SECTION("when sleeping, we use at least as much time as specified") {
+		const auto start = steady_clock::now();
+
+		const auto result = utils::run_non_interactively("sleep 1", "test");
+		REQUIRE(result == 0);
+
+		const auto finish = steady_clock::now();
+		const auto runtime = duration_cast<milliseconds>(finish - start);
+		REQUIRE(runtime.count() >= 1000);
+	}
+
+	SECTION("if program is put in background, using &, we return immediately") {
+		const auto start = steady_clock::now();
+
+		const auto result = utils::run_non_interactively("sleep 5 &", "test");
+		REQUIRE(result == 0);
+
+		const auto finish = steady_clock::now();
+		const auto runtime = duration_cast<milliseconds>(finish - start);
+		REQUIRE(runtime.count() < 5000);
+	}
+}
+
 TEST_CASE("remove_soft_hyphens remove all U+00AD characters from a string",
 	"[utils]")
 {
@@ -1674,3 +1721,247 @@ TEST_CASE("mkdir_parents() doesn't care if the path ends in a slash or not",
 		check(path + "/");
 	}
 }
+
+TEST_CASE("utf8_to_locale() converts text from UTF-8 to the encoding specified "
+	"by locale in LC_CTYPE class",
+	"[utils]")
+{
+	TestHelpers::EnvVar lc_ctype("LC_CTYPE");
+	lc_ctype.on_change([](nonstd::optional<std::string> new_charset) {
+		if (new_charset.has_value()) {
+			::setlocale(LC_CTYPE, new_charset.value().c_str());
+		} else {
+			::setlocale(LC_CTYPE, "");
+		}
+	});
+	const auto set_locale = [&lc_ctype](std::string new_locale) -> bool {
+		if (::setlocale(LC_CTYPE, new_locale.c_str()) == nullptr)
+		{
+			WARN("Couldn't set locale " + new_locale + "; test skipped.");
+			return false;
+		}
+		lc_ctype.set(new_locale);
+		return true;
+	};
+
+	SECTION("UTF-8") {
+		if (!set_locale("en_US.UTF-8")) {
+			return;
+		}
+
+		REQUIRE(utils::utf8_to_locale("") == "");
+
+		// "Просто проверяю", "Just testing" in Russian.
+		const std::string text("\xd0\x9f\xd1\x80\xd0\xbe\xd1\x81\xd1\x82\xd0"
+			"\xbe\x20\xd0\xbf\xd1\x80\xd0\xbe\xd0\xb2\xd0\xb5\xd1\x80\xd1\x8f\xd1\x8e");
+		REQUIRE(utils::utf8_to_locale(text) == text);
+	}
+
+	SECTION("KOI8-R") {
+		if (!set_locale("ru_RU.KOI8-R")) {
+			return;
+		}
+
+		REQUIRE(utils::utf8_to_locale("") == "");
+
+		// "ещё один тест", "another test" in Russian.
+		const std::string input("\xd0\xb5\xd1\x89\xd1\x91\x20\xd0\xbe\xd0\xb4"
+			"\xd0\xb8\xd0\xbd\x20\xd1\x82\xd0\xb5\xd1\x81\xd1\x82");
+		const std::string expected("\xc5\xdd\xa3\x20\xcf\xc4\xc9\xce\x20\xd4"
+			"\xc5\xd3\xd4");
+		REQUIRE(utils::utf8_to_locale(input) == expected);
+	}
+
+	SECTION("CP1251") {
+		if (!set_locale("ru_RU.CP1251")) {
+			return;
+		}
+
+		REQUIRE(utils::utf8_to_locale("") == "");
+
+		// "Приветствую!", "Greetings!" in Russian.
+		const std::string input("\xd0\x9f\xd1\x80\xd0\xb8\xd0\xb2\xd0\xb5\xd1"
+			"\x82\xd1\x81\xd1\x82\xd0\xb2\xd1\x83\xd1\x8e\x21");
+		const std::string expected("\xcf\xf0\xe8\xe2\xe5\xf2\xf1\xf2\xe2\xf3"
+			"\xfe\x21");
+		REQUIRE(utils::utf8_to_locale(input) == expected);
+	}
+}
+
+TEST_CASE("utf8_to_locale() transliterates characters unsupported by the locale's encoding",
+	"[utils]")
+{
+	TestHelpers::EnvVar lc_ctype("LC_CTYPE");
+	lc_ctype.on_change([](nonstd::optional<std::string> new_charset) {
+		if (new_charset.has_value()) {
+			::setlocale(LC_CTYPE, new_charset.value().c_str());
+		} else {
+			::setlocale(LC_CTYPE, "");
+		}
+	});
+	const auto set_locale = [&lc_ctype](std::string new_locale) -> bool {
+		if (::setlocale(LC_CTYPE, new_locale.c_str()) == nullptr)
+		{
+			WARN("Couldn't set locale " + new_locale + "; test skipped.");
+			return false;
+		}
+		lc_ctype.set(new_locale);
+		return true;
+	};
+
+	// Tests below don't spell out the expected result because different
+	// platforms might follow different transliteration rules. The best we can
+	// do is check that 1) the output is non-empty, i.e. there was no error in
+	// conversion; 2) the output is not the same as the input, i.e. the
+	// conversion did something.
+
+	SECTION("KOI8-R doesn't support Ukrainian characters") {
+		if (!set_locale("ru_RU.KOI8-R")) {
+			return;
+		}
+
+		// "Пісня", "Song" in Ukrainian.
+		const std::string input("\xd0\x9f\xd1\x96\xd1\x81\xd0\xbd\xd1\x8f");
+
+		const auto result = utils::utf8_to_locale(input);
+		REQUIRE(result != "");
+		REQUIRE(result != input);
+	}
+
+	SECTION("CP1251") {
+		if (!set_locale("ru_RU.CP1251")) {
+			return;
+		}
+
+		// "日本", "Japan" in Japanese
+		const std::string input("\xe6\x97\xa5\xe6\x9c\xac");
+
+		const auto result = utils::utf8_to_locale(input);
+		REQUIRE(result != "");
+		REQUIRE(result != input);
+	}
+}
+
+TEST_CASE("locale_to_utf8() converts text from the encoding specified by locale "
+	"in LC_CTYPE class to UTF-8",
+	"[utils]")
+{
+	TestHelpers::EnvVar lc_ctype("LC_CTYPE");
+	lc_ctype.on_change([](nonstd::optional<std::string> new_charset) {
+		if (new_charset.has_value()) {
+			::setlocale(LC_CTYPE, new_charset.value().c_str());
+		} else {
+			::setlocale(LC_CTYPE, "");
+		}
+	});
+	const auto set_locale = [&lc_ctype](std::string new_locale) -> bool {
+		if (::setlocale(LC_CTYPE, new_locale.c_str()) == nullptr)
+		{
+			WARN("Couldn't set locale " + new_locale + "; test skipped.");
+			return false;
+		}
+		lc_ctype.set(new_locale);
+		return true;
+	};
+
+	SECTION("UTF-8") {
+		if (!set_locale("en_US.UTF-8")) {
+			return;
+		}
+
+		REQUIRE(utils::locale_to_utf8("") == "");
+
+		// "Newsboat мне нравится", "I like Newsboat" in Russian.
+		const std::string text("\x4e\x65\x77\x73\x62\x6f\x61\x74\x20\xd0\xbc"
+			"\xd0\xbd\xd0\xb5\x20\xd0\xbd\xd1\x80\xd0\xb0\xd0\xb2\xd0\xb8"
+			"\xd1\x82\xd1\x81\xd1\x8f");
+		REQUIRE(utils::locale_to_utf8(text) == text);
+	}
+
+	SECTION("KOI8-R") {
+		if (!set_locale("ru_RU.KOI8-R")) {
+			return;
+		}
+
+		REQUIRE(utils::locale_to_utf8("") == "");
+
+		// "великолепная проверка", "excellent check" in Russian.
+		const std::string input("\xd7\xc5\xcc\xc9\xcb\xcf\xcc\xc5\xd0\xce\xc1"
+			"\xd1\x20\xd0\xd2\xcf\xd7\xc5\xd2\xcb\xc1");
+		const std::string expected("\xd0\xb2\xd0\xb5\xd0\xbb\xd0\xb8\xd0\xba\xd0"
+			"\xbe\xd0\xbb\xd0\xb5\xd0\xbf\xd0\xbd\xd0\xb0\xd1\x8f\x20\xd0"
+			"\xbf\xd1\x80\xd0\xbe\xd0\xb2\xd0\xb5\xd1\x80\xd0\xba\xd0\xb0");
+		REQUIRE(utils::locale_to_utf8(input) == expected);
+	}
+
+	SECTION("CP1251") {
+		if (!set_locale("ru_RU.CP1251")) {
+			return;
+		}
+
+		REQUIRE(utils::locale_to_utf8("") == "");
+
+		// "Все тесты зелёные!", "All tests green!" in Russian.
+		const std::string input("\xc2\xf1\xe5\x20\xf2\xe5\xf1\xf2\xfb\x20\xe7"
+			"\xe5\xeb\xb8\xed\xfb\xe5\x21");
+		const std::string expected("\xd0\x92\xd1\x81\xd0\xb5\x20\xd1\x82\xd0\xb5"
+			"\xd1\x81\xd1\x82\xd1\x8b\x20\xd0\xb7\xd0\xb5\xd0\xbb\xd1\x91\xd0"
+			"\xbd\xd1\x8b\xd0\xb5\x21");
+		REQUIRE(utils::locale_to_utf8(input) == expected);
+	}
+}
+
+TEST_CASE("translit() returns the value of `tocode`, maybe with \"//TRANSLIT\" appended",
+	"[utils]")
+{
+	// The behaviour of translit() is inherently platform-dependent: some
+	// platforms support transliteration for some encodings, while others
+	// support it for different encodings, or don't support it at all. This
+	// tests checks just the bare minimum: the function doesn't crash, doesn't
+	// throw exceptions, and returns something resembling a correct value.
+
+	const auto check = [](std::string fromcode, std::string tocode) {
+		DYNAMIC_SECTION(fromcode << " → " << tocode) {
+			const auto expected1(tocode);
+			const auto expected2(tocode + "//TRANSLIT");
+
+			const auto actual = utils::translit(tocode, fromcode);
+
+			INFO("Expected " << expected1);
+			INFO("      or " << expected2);
+			INFO("But got: " << actual);
+
+			// Extra parentheses required by Catch2 -- otherwise it doesn't let us use operator||.
+			REQUIRE((actual == expected1 || actual == expected2));
+		}
+	};
+
+	check("ISO-8859-1", "UTF-8");
+	check("KOI8-R", "UTF-8");
+	check("UTF-16", "UTF-8");
+	check("UTF-8", "UTF-8");
+	check("UTF-8", "UTF-16");
+	check("UTF-8", "KOI8-R");
+	check("UTF-8", "ISO-8859-1");
+}
+
+TEST_CASE("translit() always returns the same value for the same inputs",
+	"[utils]")
+{
+	const std::vector<std::string> encodings {"UTF-8", "UTF-16", "KOI8-R", "ISO-8859-1"};
+	// (fromcode, tocode) -> result
+	std::map<std::pair<std::string, std::string>, std::string> results;
+
+	for (const auto& fromcode : encodings) {
+		for (const auto& tocode : encodings) {
+			results[std::make_pair(fromcode, tocode)] = utils::translit(tocode, fromcode);
+		}
+	}
+
+	for (const auto& fromcode : encodings) {
+		for (const auto& tocode : encodings) {
+			INFO(fromcode << " → " << tocode);
+			REQUIRE(utils::translit(tocode, fromcode) == results.at(std::make_pair(fromcode, tocode)));
+		}
+	}
+}
